from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class AskChildPathBOMLinesInfo(TcBaseObj):
    """
    A set of input information for a single product structure including the associated child paths to be evaluated by
    this operation.
    
    :var clientId: Identifier used to relate partial errors associated with an instance of this information.
    :var parentBomLine: The BOM line parent of the first PS Occurrence Thread UID specified by each of the paths.
    :var useAsStable: Indicates that the child paths are specified by "stable" rather than "real" PS Occurrence Thread
    UIDs.
    :var childPaths: One or more PS Occurrence Thread child paths.
    """
    clientId: str = ''
    parentBomLine: BOMLine = None
    useAsStable: bool = False
    childPaths: List[AskChildPathBOMLinesPath] = ()


@dataclass
class AskChildPathBOMLinesPath(TcBaseObj):
    """
    One or more PS Occurrence Threads defining an input child path within a given product structure from a parent to a
    child.
    
    :var clientId: Identifier used to relate partial errors associated with an instance of this information.
    :var childPath: An ordered list of PS Occurrence Thread UIDs that specify a path from a parent
    Teamcenter::BOMLineImpl through the product structure to the child.
    """
    clientId: str = ''
    childPath: List[str] = ()


@dataclass
class AskChildPathBOMLinesResponse(TcBaseObj):
    """
    Defines the response from the askPSOccThreadChildBOMLines operation.
    
    :var output: A map of input PS Occurrence Thread UIDs to BOMLines.
    :var serviceData: The SOA framework object containing plain objects, and error information.
    """
    output: AskChildPathBOMLineMap = None
    serviceData: ServiceData = None


"""
AskChildPathBOMLineMap
"""
AskChildPathBOMLineMap = Dict[str, BOMLine]



from __future__ import annotations

from typing import List
from tcsoa.gen.Administration._2014_10.UserManagement import MakeUserResponse
from tcsoa.base import TcService


class UserManagementService(TcService):

    @classmethod
    def makeUser(cls, arguments: List[str], batchFileFmsTicket: str, enableStandardOutput: bool, enableStandardError: bool) -> MakeUserResponse:
        """
        This operation executes the make_user utility on the Teamcenter server with the specified command line
        arguments and optional batch input file. The make_user utility runs with the same user and group as the current
        session user.
        """
        return cls.execute_soa_method(
            method_name='makeUser',
            library='Administration',
            service_date='2014_10',
            service_name='UserManagement',
            params={'arguments': arguments, 'batchFileFmsTicket': batchFileFmsTicket, 'enableStandardOutput': enableStandardOutput, 'enableStandardError': enableStandardError},
            response_cls=MakeUserResponse,
        )



from __future__ import annotations

from tcsoa.gen.Ai._2006_03.Ai import ApplicationRef
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Ai._2008_06.Ai import Configuration
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetObjectsByApplicationRefsRespElem(TcBaseObj):
    """
    The objects found for the input ApplicationRefs.
    
    :var objects: A list of BusinessObject representing persistent or runtime business objects associated with the
    input ApplicationRef objects.
    :var createdWindows: A list of BOMWindow objects the client should close.
    :var additionalInfo: Reserved for future use.
    """
    objects: List[BusinessObject] = ()
    createdWindows: List[BusinessObject] = ()
    additionalInfo: AdditionalInfo2 = None


@dataclass
class GetObjectsByApplicationRefsResponse(TcBaseObj):
    """
    The objects found for the input ApplicationRefs.
    
    :var responseElements: A list of GetObjectsForApplicationRefsElement objects specifying the business objects
    associated with the input list of ConfigurationInformation objects.
    :var additionalInfo: Reserved for future use.
    :var serviceData: Partial errors
    """
    responseElements: List[GetObjectsByApplicationRefsRespElem] = ()
    additionalInfo: AdditionalInfo2 = None
    serviceData: ServiceData = None


@dataclass
class ConfigurationInfo(TcBaseObj):
    """
    Configuration and Application Ref info used to find associated BOM Lines.
    
    :var configInfo: Configuration information including BOMWindow or configuringObject.
    :var appRefs: List of ApplicationRef objects for which associated business objects are found. Using the PLMXML
    format.
    :var additionalInfo: A generic structure to be used for additional information.
    """
    configInfo: Configuration = None
    appRefs: List[ApplicationRef] = ()
    additionalInfo: AdditionalInfo2 = None


@dataclass
class AdditionalInfo2(TcBaseObj):
    """
    A generic structure to capture additional information.
    
    :var intMap: A map containing a set of (string/vector<int>) elements.
    :var dblMap: A map containing a set of (string/vector<double>) elements.
    :var strMap: A map containing a set of (string/vector<string>) elements.
    :var objMap: A map containing a set of (string/vector<businessObject>) elements.
    :var dateMap: A map containing a set of (string/vector<dateTime>) elements.
    """
    intMap: StringToIntVectorMap2 = None
    dblMap: StringToDblVectorMap2 = None
    strMap: StringToStrVectorMap2 = None
    objMap: StringToObjVectorMap2 = None
    dateMap: StringToDateVectorMap2 = None


"""
A map containing a set of (string/vector<DateTime>) elements.
"""
StringToDateVectorMap2 = Dict[str, List[datetime]]


"""
A map containing a set of (string/vector<double>) elements.
"""
StringToDblVectorMap2 = Dict[str, List[float]]


"""
A map containing a set of (string/vector<int>) elements.
"""
StringToIntVectorMap2 = Dict[str, List[int]]


"""
A map containing a set of (string/vector<BusinessObject>) elements.
"""
StringToObjVectorMap2 = Dict[str, List[BusinessObject]]


"""
A map containing a set of (string/vector<string>) elements.
"""
StringToStrVectorMap2 = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.Visualization._2008_06.DataManagement import DatasetInfo, NamedRefUploadOrUpdateInfo
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class NewSnapshot3DInput(TcBaseObj):
    """
    New input structure for creating SnapShotViewData dataset, where user can specify relation to be used and generic
    objects are used.
    
    :var clientId: Identifier that helps the client to track the objects returned.
    :var attachToInfo: SnapShotViewData attachment information
    :var datasetInfo: Dataset creation information
    :var visibleLinesList: A list of visible objects
    :var createStructureFile: If true, a static structure file is created on the server. For legacy purposes only.
    :var variantName: Variant information. It can be empty.
    :var namedRefFileUpdateInfoList: List of named reference files to be attached to SnapShotViewData dataset at time
    of creation.
    """
    clientId: str = ''
    attachToInfo: Snapshot3DAttachToInfo = None
    datasetInfo: DatasetInfo = None
    visibleLinesList: List[BusinessObject] = ()
    createStructureFile: bool = False
    variantName: str = ''
    namedRefFileUpdateInfoList: List[NamedRefUploadOrUpdateInfo] = ()


@dataclass
class Snapshot3DAttachToInfo(TcBaseObj):
    """
    Object and relation attachment information.
    
    :var attachToObject: BusinessObject input object to which SnapShotViewData dataset is to be attached to. Currently
    only ItemRevision objects are supported. 
    :var relationName: Relation name for attaching SnapShotViewData dataset
    """
    attachToObject: BusinessObject = None
    relationName: str = ''



from __future__ import annotations

from tcsoa.gen.Internal.DebugMonitor._2014_06.JournalBasedTesting import PropertyData, InitializeJBTResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class JournalBasedTestingService(TcService):

    @classmethod
    def initializeJBT(cls) -> InitializeJBTResponse:
        """
        This operation sets up the system environment and returns the site database information before Journal Based
        Testing is used. It will be called right after the login SOA, during both recording and replay phases. It will
        set the system environment variable TC_JBT=on if it has not been set, and clear all preferences whose values
        are UIDs (e.g., HistoryIDList and MRU related preferences). The operation returns the last six characters of an
        UID which uniquely identifies the site database that the client is connecting to. This operation is only used
        by JBT.
        """
        return cls.execute_soa_method(
            method_name='initializeJBT',
            library='Internal-DebugMonitor',
            service_date='2014_06',
            service_name='JournalBasedTesting',
            params={},
            response_cls=InitializeJBTResponse,
        )

    @classmethod
    def validatePropertyValues(cls, expectedPropertyValues: List[PropertyData], userEnteredErrorMessage: str) -> ServiceData:
        """
        This operation compares the property values of a list of objects against their expected values and reports an
        exception when a mismatch is found.
        
        Use cases:
        In the context of Journal Based Testing, this operation is called during the recording when the user clicks the
        OK button on the Validate Property Values dialog or when the Validate Structure Hierarchy menu is clicked. It
        compares the expected values against the actual values of the properties on the input objects. If there is any
        difference between the expected value and actual value, the validation fails and the operation throws an
        exception.
        
        Exceptions:
        >The operation will throw an exception when it finds the first property whose expected value does not match the
        actual value.
        
        214652 (error) Parameterized error message, e.g., the property item_id for object TopItem contains an expected
        value of 000258, but its actual value is 000528.
        """
        return cls.execute_soa_method(
            method_name='validatePropertyValues',
            library='Internal-DebugMonitor',
            service_date='2014_06',
            service_name='JournalBasedTesting',
            params={'expectedPropertyValues': expectedPropertyValues, 'userEnteredErrorMessage': userEnteredErrorMessage},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, WorkspaceObject, StructureContext, CfgActivityLine, CfgAttachmentLine
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from typing import Dict, List


@dataclass
class GetAttachmentLineChildrenResponse(TcBaseObj):
    """
    response of getAttachmentLineChildren method - a map of input line to it's children
    
    :var lines: map of parent attachmentline to child lines.
    :var serviceData: any partial errors to be returned.
    """
    lines: AttachmentLineToChildLinesMap = None
    serviceData: ServiceData = None


@dataclass
class GetBOMLineActivitiesResponse(TcBaseObj):
    """
    Response of getBOMLineActivities method. lines represent the activities for supplied bomline. servicesData to
    capture partialErrors.
    
    :var lines: vector of bomline to activities map
    :var serviceData: serviceData object to capture partial errors.
    """
    lines: BOMLineToActivitiesMap = None
    serviceData: ServiceData = None


@dataclass
class GetBOMLineAttachmentsResponse(TcBaseObj):
    """
    Response of getBOMLineAttachments method. lines represent the attachmentlines for supplied bomline. servicesData to
    capture partialErrors.
    
    :var lines: bomline to attachmentlines map
    :var serviceData: serviceData to capture partial errors.
    """
    lines: BOMLineToAttachmentLinesMap = None
    serviceData: ServiceData = None


@dataclass
class GetStructureContextActivityLinesResponse(TcBaseObj):
    """
    return structure of getStructureContextActivityLinesResponse. lines has the map of sc to activitylines, and
    serviceData has partial errors.
    
    :var lines: map of sc to activity lines
    :var serviceData: for reporting partial errors.
    """
    lines: StructureContextActivityLinesMap = None
    serviceData: ServiceData = None


@dataclass
class GetStructureContextTopLinesResponse(TcBaseObj):
    """
    response of getStructureContextTopLines methods. lines member has the map of StructureContext to it's toplines, and
    serviceData member has any partial errors.
    
    :var lines: map of SC to it's lines
    :var serviceData: serviceData to hold partial errors
    """
    lines: StructureContextToLinesMap = None
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateAttachmentsData(TcBaseObj):
    """
    Structure to specify input data for createOrUpdateAttachments
    
    :var bomLine: bomline - the top line of the attachment window. Optional if attLine is going to be specified.
    :var attLine: Parent attachment line-under which the new one will be created or updated. Overrides bomLine member
    if specified.
    :var objects: relation string to the workspaceobject secondaries map
    """
    bomLine: BOMLine = None
    attLine: CfgAttachmentLine = None
    objects: ContextToSecondaryObjectsMap = None


"""
map of attachmentline to it's child attachmentlines
"""
AttachmentLineToChildLinesMap = Dict[CfgAttachmentLine, List[CfgAttachmentLine]]


"""
map of bomline to it's mfg activity lines
"""
BOMLineToActivitiesMap = Dict[BOMLine, List[CfgActivityLine]]


"""
map of bomline to it's attachment lines
"""
BOMLineToAttachmentLinesMap = Dict[BOMLine, List[CfgAttachmentLine]]


"""
map of StructureContext to Activities,
"""
StructureContextActivityLinesMap = Dict[StructureContext, List[CfgActivityLine]]


"""
map of StructureContext Object to it's toplines
"""
StructureContextToLinesMap = Dict[StructureContext, List[BOMLine]]


"""
the map if relation string to the secondary workspace objects that defines the attachment(s)
"""
ContextToSecondaryObjectsMap = Dict[str, List[WorkspaceObject]]



from tcsoa.gen.UiConfig._2014_11.services import UiConfigService as imp0
from tcsoa.gen.UiConfig._2015_10.services import UiConfigService as imp1
from tcsoa.base import TcService


class UiConfigService(TcService):
    getUIConfigs = imp0.getUIConfigs
    getUIConfigs2 = imp1.getUIConfigs2
    saveUiConfigs = imp0.saveUiConfigs



from __future__ import annotations

from tcsoa.gen.ProjectManagement._2011_02.ScheduleManagement import MultiSchSpecialCopyResponse, SpecialCopyContainer
from tcsoa.base import TcService


class ScheduleManagementService(TcService):

    @classmethod
    def specialPasteScheduleTasks(cls, copyContainer: SpecialCopyContainer) -> MultiSchSpecialCopyResponse:
        """
        Gets the selected Schedule and their tasks and paste it to target task as specified by the options
        """
        return cls.execute_soa_method(
            method_name='specialPasteScheduleTasks',
            library='ProjectManagement',
            service_date='2011_02',
            service_name='ScheduleManagement',
            params={'copyContainer': copyContainer},
            response_cls=MultiSchSpecialCopyResponse,
        )



from tcsoa.gen.Cae._2012_02.services import StructureManagementService as imp0
from tcsoa.gen.Cae._2014_06.services import StructureManagementService as imp1
from tcsoa.gen.Cae._2011_06.services import SimulationProcessManagementService as imp2
from tcsoa.gen.Cae._2013_12.services import SimulationProcessManagementService as imp3
from tcsoa.base import TcService


class StructureManagementService(TcService):
    executeDatamap = imp0.executeDatamap
    executeDeriveCAEStructure = imp1.executeDeriveCAEStructure
    executeStructureMap = imp0.executeStructureMap


class SimulationProcessManagementService(TcService):
    launchSimulationTool = imp2.launchSimulationTool
    launchSimulationTool2 = imp3.launchSimulationTool2



from tcsoa.basics import TcSoaBasics
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.search_basics import TcSearchBasics


class TcUserBasics:
    @classmethod
    def find_users(cls, user_id=None, person_name=None):
        if not (user_id or person_name):
            raise ValueError('Please pass at least one of the arguments!')
        args = dict()
        if user_id:
            args['User ID'] = user_id
        if person_name:
            args['Person Name'] = person_name

        query = TcSearchBasics.get_query_by_name('Admin - Employee Information')
        matched_users = TcSearchBasics.exec_query(query, args)
        return matched_users

    @classmethod
    def get_person(cls, user: BusinessObject):
        return TcSoaBasics.getPropObject(user, 'person')

    @classmethod
    def set_person_data(
            cls,
            person: BusinessObject,
            address: str = None,
            city: str = None,
            state: str = None,
            zip_code: str = None,
            country: str = None,
            organisation: str = None,
            employee_number: str = None,
            internal_mail_code: str = None,
            mail_address: str = None,
            phone: str = None):

        attrs = dict()
        if address is not None:
            attrs['PA1'] = address
        if city is not None:
            attrs['PA2'] = city
        if state is not None:
            attrs['PA3'] = state
        if zip_code is not None:
            attrs['PA4'] = zip_code
        if country is not None:
            attrs['PA5'] = country
        if organisation is not None:
            attrs['PA6'] = organisation
        if employee_number is not None:
            attrs['PA7'] = employee_number
        if internal_mail_code is not None:
            attrs['PA8'] = internal_mail_code
        if mail_address is not None:
            attrs['PA9'] = mail_address
        if phone is not None:
            attrs['PA10'] = phone
        TcSoaBasics.setMultiPropString(cls._ensure_person(person), attrs)

    @classmethod
    def _ensure_person(cls, person: BusinessObject) -> BusinessObject:
        if person.className == 'User':
            person = cls.get_person(person)
        if person.className != 'Person':
            raise ValueError('Please pass a User or Person object')
        return person

    @classmethod
    def get_person_address(cls, person: BusinessObject) -> str:
        return TcSoaBasics.getPropString(cls._ensure_person(person), 'PA1')

    @classmethod
    def get_person_city(cls, person: BusinessObject) -> str:
        return TcSoaBasics.getPropString(cls._ensure_person(person), 'PA2')

    @classmethod
    def get_person_state(cls, person: BusinessObject) -> str:
        return TcSoaBasics.getPropString(cls._ensure_person(person), 'PA3')

    @classmethod
    def get_person_zip_code(cls, person: BusinessObject) -> str:
        return TcSoaBasics.getPropString(cls._ensure_person(person), 'PA4')

    @classmethod
    def get_person_country(cls, person: BusinessObject) -> str:
        return TcSoaBasics.getPropString(cls._ensure_person(person), 'PA5')

    @classmethod
    def get_person_organisation(cls, person: BusinessObject) -> str:
        return TcSoaBasics.getPropString(cls._ensure_person(person), 'PA6')

    @classmethod
    def get_person_employee_number(cls, person: BusinessObject) -> str:
        return TcSoaBasics.getPropString(cls._ensure_person(person), 'PA7')

    @classmethod
    def get_person_internal_mail_code(cls, person: BusinessObject) -> str:
        return TcSoaBasics.getPropString(cls._ensure_person(person), 'PA8')

    @classmethod
    def get_person_mail_address(cls, person: BusinessObject) -> str:
        return TcSoaBasics.getPropString(cls._ensure_person(person), 'PA9')

    @classmethod
    def get_person_phone(cls, person: BusinessObject) -> str:
        return TcSoaBasics.getPropString(cls._ensure_person(person), 'PA10')



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ImportExportOptionsInfo(TcBaseObj):
    """
    The 'ImportExportOptionsInfo' structures holds the name/value pairs of export option(s). Value can be single or
    multiple valued. The export options influences the business object export and has default value.
    
    :var optionName: Export option name for remote export.
    :var optionValue: Values for remote export option (can be multi-valued strings).
    """
    optionName: str = ''
    optionValue: List[str] = ()


@dataclass
class RemoteExportInfo(TcBaseObj):
    """
    The 'RemoteExportInfo' holds the business object and related information meant for remote export.
    
    :var object: Business object for remote export operation.
    :var reason: Reason for exporting business object. Its not mandatory to pass any reason, empty string can be passed.
    :var targetSites: List of sites, where business object export will be performed.
    :var remoteCheckout: Along with remote export, if the remote check out needs to be performed, pass true. Only one
    target site should be provided with true value.
    :var checkoutId: Checkout identifier(empty string can be passed).
    :var exportOptions: List of options to influence the business object export.
    """
    object: BusinessObject = None
    reason: str = ''
    targetSites: List[str] = ()
    remoteCheckout: bool = False
    checkoutId: str = ''
    exportOptions: List[ImportExportOptionsInfo] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Cfg0ConfiguratorPerspective
from tcsoa.gen.Internal.Configurator._2018_06.ConfiguratorManagement import MatrixCell, GetVariabilityResponse, ExpressionMatrix, GetConfigurationSessionInfoResponse, ValidateProductConditionResponse
from typing import List
from tcsoa.base import TcService


class ConfiguratorManagementService(TcService):

    @classmethod
    def getConfigurationSessionInfo(cls, sessionConfigObjs: List[BusinessObject]) -> GetConfigurationSessionInfoResponse:
        """
        This operation retrieves the Product Configurator configuration session information from the objects that are
        marked to carry such information. User starts a Product Configurator session to create product configurations
        using configuration features. The session holds product information such as the, Cfg0ConfContext, RevisionRule,
        Effectivity, Rule Date. The product information is also represented as Cfg0ConfiguratorPerspective object. The
        product information configures the features  which user can select to build product configuration. The session
        also holds parameters called as Configuration Profile which govern the evaluation of configurator constraints
        during validation and expansion of the product configuration. The objects must implement ConfigurationSession
        behavior, so that Product Configurator configuration session information can be attached and retrieved from
        them.
        
        Use cases:
        User wants to view a Configurator Criteria which has the Product Configurator configuration session information
        attached. User may use this information to apply the product configuration associated with the Configurator
        Criteria on product content and configure it. The session information will recreate the Product Configurator
        configuration session with features and constraints against which the product content configuration will be
        evaluated.
        """
        return cls.execute_soa_method(
            method_name='getConfigurationSessionInfo',
            library='Internal-Configurator',
            service_date='2018_06',
            service_name='ConfiguratorManagement',
            params={'sessionConfigObjs': sessionConfigObjs},
            response_cls=GetConfigurationSessionInfoResponse,
        )

    @classmethod
    def getVariability(cls, configuratorPerspectives: List[Cfg0ConfiguratorPerspective]) -> GetVariabilityResponse:
        """
        This operation returns variability for list of input Configurator perspectives. The variability can be all
        configured model families, models, groups, families and values based on the corresponding Configurator Context
        Item, Revision Rule, Effectivity and Rule date in the input Configurator perspectives. If an allocated object
        is a member of a group it will be returned as grouped variability. Otherwise it will be returned as ungrouped
        variability.
        
        Use cases:
        The getVariability operation should be invoked to fetch variability such as all configured model families,
        models, groups, families and values based on the corresponding Configurator Context Item, Revision Rule,
        Effectivity and Rule date in the input Configurator perspective.
        
        Example:
        Consider the following Variability data in a Configurator context:
        Car_Product_Context
        Models
            ATS
            CTS
        Group
            Engine (Cfg0LiteralValueFamily) - Released
                2.0L Turbo (Cfg0LiteralOptionValue) - Released   Effectivity: 01-Jan-2021&hellip;31-Dec-2030
                3.0L (Cfg0LiteralOptionValue) - Released   Effectivity: 01-Jan-2021&hellip;31-Dec-2030
                SumValue1
                3.0L Turbo (Cfg0LiteralOptionValue) - Released   Effectivity: 01-Jan-2021&hellip;31-Dec-2030
                SumValue1
            Fuel (Cfg0LiteralValueFamily) - Working
                Diesel (Cfg0LiteralOptionValue) - Working   Effectivity: 01-Jan-2021&hellip;31-Dec-2030
                Electrical (Cfg0LiteralOptionValue) - Working   Effectivity: 01-Jan-2021&hellip;31-Dec-2030
            Transmission (Cfg0LiteralValueFamily) - Released
                Automatic (Cfg0LiteralOptionValue) - Released   Effectivity: 01-Jan-2021&hellip;31-Dec-2030
                Manual (Cfg0LiteralOptionValue) - Released   Effectivity: 01-Jan-2026&hellip;31-Dec-2030
            FeatureColor (Cfg0FeatureFamily) - Released
                Red (Cfg0Feature) - Released   Effectivity: 01-Jan-2021&hellip;31-Dec-2030
                Green (Cfg0Feature) - Released   Effectivity: 01-Jan-2026&hellip;31-Dec-2030
            StandAloneFeature (Cfg0Feature)
                StandAloneFeature (Cfg0Feature) - Released   Effectivity: 01-Jan-2021&hellip;31-Dec-2030
        Unassigned Families
            FeatureSet (Cfg0FeatureSet) - Released
                StandAloneFeature (Cfg0Feature) - Released   Effectivity: 01-Jan-2021&hellip;31-Dec-2030
                Green (Cfg0Feature) - Released   Effectivity: 01-Jan-2021&hellip;31-Dec-2030
            SummaryFamily (Cfg0SummaryOptionFamily) - Released
                SumValue1 (Cfg0SummaryOptionValue) - Released   Effectivity: 01-Jan-2021&hellip;31-Dec-2030
            PackageFamily (Cfg0PackageOptionFamily) - Released
                PackageValue1 (Cfg0PackageOptionValue) - Released   Effectivity: 01-Jan-2021&hellip;31-Dec-2030
                        Electrical
                        Green
        Use case 1:
        &#61656;   Configurator Context Item : Car_Product_Context
        &#61656;   Revision Rule: Any Status; Working
        &#61656;   Effectivity: No Effectivity
        &#61656;   Rule date: System Default
        The output of getVariability operation will be all the variability mentioned above.
        Use case 2:
        &#61656;   Configurator Context Item: Car_Product_Context
        &#61656;   Revision Rule: Any Status; Working
        &#61656;   Effectivity: 01-Jan-2020&hellip;31-Dec-2025
        &#61656;   Rule date: System Default
        The output of getVariability operation will be all the variability mentioned above except Manual value of
        Transmission family and Green feature of FeatureColor family, because their effectivity is not present in the
        input effectivity criteria.
        Use case 3:
        &#61656;   Configurator Context Item: Car_Product_Context
        &#61656;   Revision Rule: Any Status; No Working
        &#61656;   Effectivity: No Effectivity
        &#61656;   Rule date: System Default
        The output of getVariability operation will be all the variability mentioned above except Fuel family and its
        Diesel and Electrical option values because they are not released and input Revision Rule is Any Status; No
        Working.
        """
        return cls.execute_soa_method(
            method_name='getVariability',
            library='Internal-Configurator',
            service_date='2018_06',
            service_name='ConfiguratorManagement',
            params={'configuratorPerspectives': configuratorPerspectives},
            response_cls=GetVariabilityResponse,
        )

    @classmethod
    def validateProductConditions(cls, sessionConfigObj: BusinessObject, expressionMatrix: ExpressionMatrix, matrixSubset: List[MatrixCell]) -> ValidateProductConditionResponse:
        """
        This operation process cells of Expression matrix expressionMatrix to validate satisfiability of Expression(s)
        against optional scope Expression(s). Expression matrix row(s) represents Expressions(s) to validate and
        column(s) represent scope Expression(s). So each cell in the Expression matrix represents a combination of two
        expressions, one is referred to as an "Expression to validate" the other as a "Scope Expressions".
        
        Cell(s) of the Expression matrix to be process for validation operation are determined with matrixSubset which
        is a list of cells of Expression matrix. Validation operation also does the application of the constraint rules
        such as inclusion rule, exclusion rule or availability rule for the Configurator Perspective.
        
        This operation retrieves the Product Configurator configuration session information from the sessionConfigObj
        object that is marked to carry such information. A operation user starts a Product Configurator session to
        create product configurations using configuration features. The session holds product information such as the,
        Cfg0ConfContext, RevisionRule, Effectivity, Rule Date. The product information is also represented as
        Cfg0ConfiguratorPerspective object. The product information configures the features which user can select to
        build product configuration. The session also holds parameters called as Configuration Profile which govern the
        evaluation of configurator constraints during validation and expansion of the product configuration.
        sessionConfigObj must implement ConfigurationSession behavior, so that Product Configurator configuration
        session information can be attached and retrieved from them. For example operation caller can use VariantRule
        or Cfg0AbsConfiguratorCriteria type of objects to which they can attach required Configuration session
        information.
        The operation response conveys the information about the status of the configuration and independent set of
        constraint rules which are responsible together for conflict to occur.
        
        Use cases:
        Consider the following Variability data and Rules in a Configurator context:
        
        Variability:
        Models  
         ATS 
         CTS 
         SRX 
         Sedan 
        
        Trim 
         Luxury
         Performance
         Premium
        Standard
        
        DriveTrain 
        RearWheel
        xDrive
        
        Engine 
         2.0L Turbo
         3.0L
         3.0L Turbo
        
        Fuel 
         Diesel
         Electrical
         Gasoline
         Hybrid
         
        Transmission 
         Automatic
         Manual
        
        ABS 
         3Wheel
         4Wheel
        
        FrontBreak 
         Disc
         Drum
        
        RearBreak 
         Disc
         Drum
        
        Wheel 
         17"
         18"
         19"
        
        Rules:
        Include Rule1 ATS Model--> Wheel Type = 18"
        Include Rule2 Performance Trim Type -> Transmission = Manual
        Include Rule3   ATS Model--> Trim Type = Performance"
        All the families are non-discretionary families.
        
        Use Case1:
        expressionMatrix populated with Expression To Validate as '(Transmission=Automatic OR Manual) AND Wheel=19"'
        and 
        Scope Expression as &lsquo;Model=ATS'.  In-order to perform validation, matrixSubset need to have an entry as
        (0,0) as index for first expression to validate is 0 and scope expression index is 0.
        
        There are violations and parameter criteriaStatus has value inValid.
        The output of the operation will contain two set of violation lists.
        
        First list will contain Include Rule1 'ATS Model--> Wheel Type = 18"' and
        
        second list will contain following rules 
        Include Rule2 Performance Trim Type -> Transmission = Manual
        Include Rule3   ATS Model--> Trim Type = Performance"
        
        Looking at two set of violation List user will be able to isolate the problem.
        """
        return cls.execute_soa_method(
            method_name='validateProductConditions',
            library='Internal-Configurator',
            service_date='2018_06',
            service_name='ConfiguratorManagement',
            params={'sessionConfigObj': sessionConfigObj, 'expressionMatrix': expressionMatrix, 'matrixSubset': matrixSubset},
            response_cls=ValidateProductConditionResponse,
        )



from __future__ import annotations

from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GenerateContextIDsInput2(TcBaseObj):
    """
    This structure is the input sructure for the generateContextSpecificIDs service.
    This contains the informanion of context name and number of IDs to be generated for that context name.
    
    :var clientID: A unique string supplied by the caller. This ID is used to identify returned
    GenerateContextIDResponse elements and Partial Errors associated with this input GenerateContextIDsInput2.
    :var contextName: Name of the context for which IDs to be generated. A context name is a string that can be up to
    256 characters long. This string should not be empty, in order to generate IDs, an error is returned if user tries
    to generate ID for a empty context name.
    :var contextTypeName: The name of the business object for which the property values are to be generated and
    validated. An error 39007 is returned by the operation if the type name is not valid.
    :var contextPropName: Property name associated with the Context Name. This property name will be used to validate
    ID uniqueness.
    :var validateIDs: If true, the IDs should be validated for uniquness, otherwise no validation will be done. If
    false, contextType and contextPropName can be NULL.
    :var numberOfIDs: The number of IDs to be generated for a given context name. This is a mandatory field and should
    not be 0 or any negative number to generate IDs. An error is returned, if user tries to generate ID for a context
    name with invalid value (0 or negative number) for this field.
    """
    clientID: str = ''
    contextName: str = ''
    contextTypeName: str = ''
    contextPropName: str = ''
    validateIDs: bool = False
    numberOfIDs: int = 0



from __future__ import annotations

from tcsoa.gen.BusinessObjects import WorkspaceObject, EPMTaskTemplate
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetWorkflowTemplatesInputInfo(TcBaseObj):
    """
    Structure to define input for workflow Templates
    
    :var clientId: Client Id.
    :var includeUnderConstruction: If set to true, the operation will return Under construction templates for
    administrative users only.  Otherwise only available workflow templates are returned.
    :var getFiltered: If set to true, the operation will return assigned or filtered list of workflow templates based
    on Target Objects and Object Types.
    :var targetObjects: List of target objects to be used for getting the filtered list of workflow templates.
    :var objectTypes: List of target object types to be used for getting the filtered list of workflow templates. This
    argument is not required if targetObjects are specified.
    :var group: User group to get the filtered list of  workflow templates.
    """
    clientId: str = ''
    includeUnderConstruction: bool = False
    getFiltered: bool = False
    targetObjects: List[WorkspaceObject] = ()
    objectTypes: List[str] = ()
    group: str = ''


@dataclass
class GetWorkflowTemplatesOutput(TcBaseObj):
    """
    Structure to define output for workflow Templates
    
    :var clientId: Client Id.
    :var workflowTemplates: A list of output workflow Templates.
    """
    clientId: str = ''
    workflowTemplates: List[EPMTaskTemplate] = ()


@dataclass
class GetWorkflowTemplatesResponse(TcBaseObj):
    """
    Structure to define response for workflow Templates
    
    :var serviceData: ServiceData contains any partial errors.
    :var templatesOutput: Complete list of workflow Templates Output.
    """
    serviceData: ServiceData = None
    templatesOutput: List[GetWorkflowTemplatesOutput] = ()



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.StructureManagement._2008_03.Structure import ReSequenceParameter
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class StructureService(TcService):

    @classmethod
    def resequence(cls, input: List[ReSequenceParameter]) -> ServiceData:
        """
        Re-Sequence the lines with the start number and the increment number with option to re-sequence their children
        by using predecessor relationship.
        
        Use cases:
        User has a large structure that has been restructured so that the find no values are not well organized. User
        invokes the operation to re-sequence the lines.
        
        """
        return cls.execute_soa_method(
            method_name='resequence',
            library='Internal-StructureManagement',
            service_date='2008_03',
            service_name='Structure',
            params={'input': input},
            response_cls=ServiceData,
        )



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FetchODSResponse(TcBaseObj):
    """
    The return structure for the fetchODSRecords operation.
    
    :var odsResults: The results structure for the fetchODSRecords operation.
    :var serviceData: The ServiceData returns partial failure information.
    """
    odsResults: FetchODSResults = None
    serviceData: ServiceData = None


@dataclass
class FetchODSResults(TcBaseObj):
    """
    Stores the returned data from fetchODSRecords operation.
    
    :var fmsTickets: A list of FMS read tickets for the TC XML output files.
    :var systemLogFiles: A list of the names of the remote system log files used to assist in troubleshooting.
    :var failedObjects: A list of failed Business Objects UIDs.
    :var odsSessionOutput: A map (string, list of strings) of data returned from the current ODS session.
    """
    fmsTickets: List[str] = ()
    systemLogFiles: List[str] = ()
    failedObjects: List[str] = ()
    odsSessionOutput: StringVectorMap = None


"""
The string key to a vector of string values map.
"""
StringVectorMap = Dict[str, List[str]]



from __future__ import annotations

from typing import List
from tcsoa.gen.Query._2014_11.Finder import ObjectsGroupedByPropertyResponse, ObjectPropertyGroupInput, SearchResponse2, SearchInput2
from tcsoa.base import TcService


class FinderService(TcService):

    @classmethod
    def groupObjectsByProperties(cls, objectPropertyGroupInputList: List[ObjectPropertyGroupInput]) -> ObjectsGroupedByPropertyResponse:
        """
        This operation classifies business objects into groups. In the list of ObjectPropertyGroupInput objects, each
        object containing an internal property name, list of PropertyGroupingValue objects identifying groups and a
        list of business objects to be classified into the groups. Each PropertyGroupingValue object contains a start
        and an end value. The end value is used for range values if populated. Unclassified business objects are
        retuned back in a list.
        
        Use cases:
        In Active Workspace client, a user navigates to a new filter category on the filter panel. The cells in the
        visible view for the search results (e.g. list view or table view) need to be colored to match the colors on
        the bar chart for the objects in the search results. The client invokes this operation, which identifies the
        property group (or bar on the chart) the objects belong to. The client can then color the cells corresponding
        to the objects appropriately based on the grouping information that is returned.
        """
        return cls.execute_soa_method(
            method_name='groupObjectsByProperties',
            library='Query',
            service_date='2014_11',
            service_name='Finder',
            params={'objectPropertyGroupInputList': objectPropertyGroupInputList},
            response_cls=ObjectsGroupedByPropertyResponse,
        )

    @classmethod
    def performSearch(cls, searchInput: SearchInput2) -> SearchResponse2:
        """
        This operation routes search request to a specific provider specified as providerName in the searchInput
        structures. 
        A framework allows custom solution to be able to provide a new specific provider to collect data, perform
        sorting and filtering. Such provider can be a User Inbox retriever, or Full Text searcher. The new data
        provider can be encapsulated via a new runtime business object from Fnd0BaseProvider class. The implementation
        is done using its fnd0performSearch operation.
        
        RuntimeBusinessObject
        ---- Fnd0BaseProvider (foundation template)
        -------- Fnd0GetChildrenProvider(foundation template)
        -------- Awp0FullTextSearchProvider (aws template)
        -------- Awp0TaskSearchProvider (aws template)
        -------- Aos0TestProvider (aosinternal template)
        -------- etc.
        
        This operation provides a common interface to send the request to and receive the response from a new data
        provider. Ultimately it allows common framework in UI to support filter, pagination, and sorting. 
        This operation allows user to send the search input, filter values, and sorting data.  These input values then
        be passed to the fnd0performSearch operation input values, which then use to collect, sort, and filter its
        results.
        The first two input parameters are important.  The first input is provider name.  This is a string to represent
        the type name of RuntimeBusinessObject to which this request should be routed to.  If the template that
        contains the class is not installed, a partial error 217016 is returned. The second input is the search input
        map.  The key is different per each provider.  For example, for Full Text searcher, the input key would be
        searchString.  For User Inbox retriever, the input key would be Inbox Type.  The fnd0performSearch
        implementation for each provider shall take into account on the key name as it is used to store the values in
        OperationInput object.
        """
        return cls.execute_soa_method(
            method_name='performSearch',
            library='Query',
            service_date='2014_11',
            service_name='Finder',
            params={'searchInput': searchInput},
            response_cls=SearchResponse2,
        )



from __future__ import annotations

from tcsoa.gen.Internal.VendorManagementAW._2020_12.VendorManagement import OrderedViewModelInput
from tcsoa.base import TcService
from tcsoa.gen.Internal.VendorManagementAW._2019_12.VendorManagement import ViewModelRowsResponse


class VendorManagementService(TcService):

    @classmethod
    def sortAndFilterViewModelRows(cls, input: OrderedViewModelInput) -> ViewModelRowsResponse:
        """
        This operation returns the filtered and sorted logical rows for a given row&rsquo;s, filter and sort criteria.
        """
        return cls.execute_soa_method(
            method_name='sortAndFilterViewModelRows',
            library='Internal-VendorManagementAW',
            service_date='2020_12',
            service_name='VendorManagement',
            params={'input': input},
            response_cls=ViewModelRowsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Awb0Element, Awb0ProductContextInfo, WorkspaceObject
from typing import List
from tcsoa.gen.Internal.ActiveWorkspaceBom._2017_12.OccurrenceManagement import OccurrenceCursor, OccurrenceInfo6
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class OccurrenceSortInput(TcBaseObj):
    """
    OccurrenceSortInput specifies the internal property name and sorting order to sort the occurrences. Sorting order
    can be "ASC" or "DESC".
    
    :var propertyName: Internal property name on which sorting is to be done.
    :var sortingOrder: Order of sorting e.g. "ASC" or "DESC".
    """
    propertyName: str = ''
    sortingOrder: str = ''


@dataclass
class ParentChildrenInfo(TcBaseObj):
    """
    ParentChildrenInfo specifies parent and its children information. The parent and children are associated through
    PSBOMViewRevision and PSOccurrence objects.
    
    :var parentInfo: Information of parent occurrence.
    :var childrenInfo: Information of children occurrences corresponding to parentInfo.
    :var cursor: Cursor that represents point to which the results have been fetched. This field is used for subsequent
    calls.
    """
    parentInfo: OccurrenceInfo6 = None
    childrenInfo: List[OccurrenceInfo6] = ()
    cursor: OccurrenceCursor = None


@dataclass
class RemoveSubstitutesData(TcBaseObj):
    """
    Contains element (BusinessObject) for which input substitute objects (WorkspaceObject) are to be removed.
    
    :var element: The object for which substitute objects are to be removed. Valid types are PartUsage,
    PartBreakdownElement, Awb0DesignElement, Fgb0PartUsage, Fgb0PartUsageLine, Fgb0PartBreakdownElement, etc.
    :var substitutesToBeRemoved: A list of substitutes to be removed. Valid types are : ItemRevision, Bom0Part.
    :var productContextInfo: Business Object of type Awb0ProductContextInfo that contains the product as well as all
    the configuration information.
    """
    element: BusinessObject = None
    substitutesToBeRemoved: List[WorkspaceObject] = ()
    productContextInfo: Awb0ProductContextInfo = None


@dataclass
class AddSubstitutesData(TcBaseObj):
    """
    Contains the element (BusinessObject) and list of substitutes to be added (WorkspaceObject).
    
    :var element: The object for which new substitutes are to be added. Valid types are PartUsage,
    PartBreakdownElement, Awb0DesignElement, Fgb0PartUsage, Fgb0PartUsageLine, Fgb0PartBreakdownElement, etc.
    :var substitutesToBeAdded: A list of objects to be added as substitutes. Valid types are ItemRevision, Bom0Part.
    """
    element: BusinessObject = None
    substitutesToBeAdded: List[WorkspaceObject] = ()


@dataclass
class DefaultName(TcBaseObj):
    """
    Contains the pattern to form the new IDs of the input Awb0Element objects.
    
    :var autogen: If true, IDs generated automatically; otherwise, not.
    :var prefix: The prefix to be attached to the ID. This field is ignored if autogen is true.
    :var suffix: The suffix to be attached to the ID. This field is ignored if autogen is true.
    :var fromString: The string to be replaced by "toString" variable. This field is ignored if autogen is true.
    :var toString: The string which replaces the fromString in the ID of the object. This field is ignored if autogen
    is true.
    """
    autogen: bool = False
    prefix: str = ''
    suffix: str = ''
    fromString: str = ''
    toString: str = ''


@dataclass
class DuplicateAndReplaceData(TcBaseObj):
    """
    Contains all the information needed to duplicate the input structures.
    
    :var productContextInfo: The Awb0ProductContextInfo object containing configuration information.
    :var elements: The selected Awb0Element objects to be duplicated and replaced with new component. The selection can
    be a leaf element or a sub assembly.
    :var defaultName: Contains the pattern to form the new IDs of the input Awb0Element objects.
    :var duplicateFlags: A bitmap for the duplicate flags set.
    1 &ndash; Smart selection using TopLine assigned projects
    2 &ndash; Rename CAD files
    4 &ndash; Return cloned object map information
    8 &ndash; run duplicate in Background mode
    16 &ndash; run duplicate in validate mode.
    """
    productContextInfo: Awb0ProductContextInfo = None
    elements: List[Awb0Element] = ()
    defaultName: DefaultName = None
    duplicateFlags: int = 0



import os
from dataclasses import dataclass
from typing import List, Optional


class FMSException(Exception):
    """ Exception which will have the Native Exception as inner Exception """
    pass


@dataclass
class FccGetLastErrorResponse:
    return_code: int
    error_msg: str


@dataclass
class FccRegisterTicketsResponse:
    ticket_uids: List[str]
    ifails: List[Optional[int]]


@dataclass
class FccUnRegisterTicketsResponse:
    return_code: int
    ifails: List[Optional[int]]


@dataclass
class FccDownloadFilesFromPlmResponse:
    local_files: List[str]
    ifails: List[Optional[int]]


@dataclass
class FccDownloadRenderingFilesResponse:
    return_code: int
    results: List[str]
    ifails: List[Optional[int]]


@dataclass
class FccDownloadFilesToLocationResponse:
    ifails: List[Optional[int]]


@dataclass
class UploadFilesToPlmResponse:
    volume_ids: List[str]
    ifails: List[Optional[int]]


@dataclass
class RollbackFilesUploadedToPlmResponse:
    ifails: List[Optional[int]]


def FmsExceptionConv(func):
    def inner(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as ex:
            raise FMSException('A C# Exception was thrown, see inner Exception') from ex
    return inner


class FccClientProxy:
    def __init__(self):
        import clr
        dll_location = self._determine_dll_location()
        clr.AddReference(dll_location)
        from Teamcenter.FMS.FCCProxy.ClientCache import NetFileCacheProxy
        self.dll = NetFileCacheProxy()

    def _determine_dll_location(self) -> str:
        dll_location = None
        if 'FMS_HOME' in os.environ:
            dll_location = os.path.join(os.environ['FMS_HOME'], 'lib', 'FCCNetClientProxy4064.dll')
        if 'TCSOA_FSC_NET_DLL' in os.environ:
            dll_location = os.environ['TCSOA_FSC_NET_DLL']
        if not dll_location or not os.path.exists(dll_location):
            raise FMSException('FSC DLL could not be found! Please set either FMS_HOME or TCSOA_FSC_NET_DLL environment var!')
        return dll_location

    @FmsExceptionConv
    def fcc_set_locale(self, locale: str):
        self.dll.SetLocale(locale)

    @FmsExceptionConv
    def fcc_register_tickets(self, tickets: List[str]) -> FccRegisterTicketsResponse:
        uids, ifails = self.dll.RegisterTickets(tickets, [], [])
        return FccRegisterTicketsResponse(list(uids), list(ifails))

    @FmsExceptionConv
    def fcc_unregister_tickets(self, uids: List[str]) -> FccUnRegisterTicketsResponse:
        ifails = self.dll.UnRegisterTickets(uids, [])
        return FccUnRegisterTicketsResponse(0, list(ifails))

    @FmsExceptionConv
    def fcc_download_file_from_plm(self, policy: str, uid: str, cb: Optional[object], client_object: Optional[object]) -> str:
        local_file = self.dll.DownloadFileFromPLM(policy, uid, cb, client_object)
        return local_file

    @FmsExceptionConv
    def fcc_download_files_from_plm(self, policy: str, uids: List[str], cb: Optional[object], client_object: Optional[object]) -> FccDownloadFilesFromPlmResponse:
        local_files, ifails = self.dll.DownloadFilesFromPLM(policy, uids, cb, client_object, [], [])
        return FccDownloadFilesFromPlmResponse(list(local_files), list(ifails))

    @FmsExceptionConv
    def fcc_download_file_to_location(self, policy: str, uid: str, cb: Optional[object], client_object: Optional[object], target_dir: str, filename: str):
        self.dll.DownloadFileToLocation(policy, uid, cb, client_object, target_dir, filename)

    @FmsExceptionConv
    def fcc_download_files_to_location(self, policy: str, uids: List[str], cb: Optional[object], client_object: Optional[object], target_dir: str, file_paths: List[str]) -> FccDownloadFilesToLocationResponse:
        ifails = self.dll.DownloadFilesToLocation(policy, uids, cb, client_object, target_dir, file_paths, [])
        return FccDownloadFilesToLocationResponse(list(ifails))

    @FmsExceptionConv
    def fcc_upload_file_to_plm(self, uid: str, cb: Optional[object], client_object: Optional[object], file_path: str) -> str:
        volume_id = self.dll.UploadFileToPLM(uid, cb, client_object, file_path)
        return volume_id

    @FmsExceptionConv
    def fcc_upload_files_to_plm(self, uids: List[str], cb: Optional[object], client_object: Optional[object], file_paths: List[str]) -> UploadFilesToPlmResponse:
        ret_val, volume_ids, ifails = self.dll.UploadFilesToPLM(uids, cb, client_object, file_paths, True, [], [])
        return UploadFilesToPlmResponse(list(volume_ids), list(ifails))

    @FmsExceptionConv
    def fcc_rollback_file_uploaded_to_plm(self, uid: str, volume_id: str):
        self.dll.RollbackFileUploadedToPLM(uid, volume_id)

    @FmsExceptionConv
    def fcc_rollback_files_uploaded_to_plm(self, uids: List[str], volume_ids: List[str]) -> RollbackFilesUploadedToPlmResponse:
        ifails = self.dll.RollbackFilesUploadedToPLM(uids, volume_ids, [])
        return RollbackFilesUploadedToPlmResponse(list(ifails))



from __future__ import annotations

from tcsoa.gen.Configuration._2010_04.SmartUiBldr import GetValuesForComponentResponse, ComponentInfo, ForTypePreferences, ListObjectsForComponentsResponse, CurrentSelectionsMap, ListForerunnersForComponentsResponse, GetPropertiesForSelectionsResponse, ListValuesForComponentsResponse, GetResultForSelectionsResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class SmartUiBldrService(TcService):

    @classmethod
    def getPropertiesForSelections(cls, rootComponent: ComponentInfo, forTypePref: ForTypePreferences, currentSelections: CurrentSelectionsMap, container: str) -> GetPropertiesForSelectionsResponse:
        """
        As per the given Smart Code configuration, the getPropertiesForSelections operation is used to get the Form
        properties and their corresponding values.  
        
        For example if 'Component Type' is mapped to 'user_data_1' on Item Master Form through FieldSet,
        getPropertiesForSelections method returns a map containing ( 'user_data_1', 'BA (component part)' ) as an
        entry. This map contains as many elements as the number of FieldSet entries in Smart Code configuration.
        """
        return cls.execute_soa_method(
            method_name='getPropertiesForSelections',
            library='Configuration',
            service_date='2010_04',
            service_name='SmartUiBldr',
            params={'rootComponent': rootComponent, 'forTypePref': forTypePref, 'currentSelections': currentSelections, 'container': container},
            response_cls=GetPropertiesForSelectionsResponse,
        )

    @classmethod
    def getResultForSelections(cls, rootComponent: ComponentInfo, currentSelections: CurrentSelectionsMap, alsoGetMappedProperties: bool) -> GetResultForSelectionsResponse:
        """
        The getResultForSelections operation is used to get the Smart Code for Item ID as per the given Smart Code
        configuration.
        For example for the below mentioned selections, final ItemID - 300005190 is generated using
        getResultForSelections::
        { "ITEM_COMP_TYPE","BA(Component part)", "ITEM_MACHINE_TYPE","A-Type",
         "ITEM_COUNTER_1","300005", "ITEM_DIMENSION","1",  "ITEM_DIMENSION_RW","190"   }
        """
        return cls.execute_soa_method(
            method_name='getResultForSelections',
            library='Configuration',
            service_date='2010_04',
            service_name='SmartUiBldr',
            params={'rootComponent': rootComponent, 'currentSelections': currentSelections, 'alsoGetMappedProperties': alsoGetMappedProperties},
            response_cls=GetResultForSelectionsResponse,
        )

    @classmethod
    def getValuesForComponent(cls, component: List[ComponentInfo], rootComponent: ComponentInfo, currentSelections: CurrentSelectionsMap) -> GetValuesForComponentResponse:
        """
        For a given set of user selections, getValuesForComponents returns the next available Smart Code value/string
        in the database for the given Component. 
        For Example, if previously assigned Item ID's Smart Code is -  "300010", getValuesForComponents returns
        "300011" as the next available Smart Code value for Item ID for the given component.
        """
        return cls.execute_soa_method(
            method_name='getValuesForComponent',
            library='Configuration',
            service_date='2010_04',
            service_name='SmartUiBldr',
            params={'component': component, 'rootComponent': rootComponent, 'currentSelections': currentSelections},
            response_cls=GetValuesForComponentResponse,
        )

    @classmethod
    def listForerunnersForComponents(cls, components: List[ComponentInfo], currentSelections: CurrentSelectionsMap) -> ListForerunnersForComponentsResponse:
        """
        As per the given Smart Code configuration, the listForerunnersForComponents operation is used to get all the
        components implicitly dependent on each of the given component.
        """
        return cls.execute_soa_method(
            method_name='listForerunnersForComponents',
            library='Configuration',
            service_date='2010_04',
            service_name='SmartUiBldr',
            params={'components': components, 'currentSelections': currentSelections},
            response_cls=ListForerunnersForComponentsResponse,
        )

    @classmethod
    def listObjectsForComponents(cls, compInfo: List[ComponentInfo], typePref: ForTypePreferences, currSelection: CurrentSelectionsMap) -> ListObjectsForComponentsResponse:
        """
        The listObjectsForComponents operation  is used to get the components explicitly dependent on each of the given
        components as per the Smart Code configuration set by the user.
        
        For example if we pass 'Component Type' as the input component, its explicit dependents 'Machine Type' will be
        returned as output components.
        """
        return cls.execute_soa_method(
            method_name='listObjectsForComponents',
            library='Configuration',
            service_date='2010_04',
            service_name='SmartUiBldr',
            params={'compInfo': compInfo, 'typePref': typePref, 'currSelection': currSelection},
            response_cls=ListObjectsForComponentsResponse,
        )

    @classmethod
    def listValuesForComponents(cls, compInfo: List[ComponentInfo], typePref: ForTypePreferences, currSelection: CurrentSelectionsMap) -> ListValuesForComponentsResponse:
        """
        The  listValuesForComponents operation is used to get the Smart Code values for Item IDs that each of the given
         component  can assume as per the Smart Code configuration set by the user.
        """
        return cls.execute_soa_method(
            method_name='listValuesForComponents',
            library='Configuration',
            service_date='2010_04',
            service_name='SmartUiBldr',
            params={'compInfo': compInfo, 'typePref': typePref, 'currSelection': currSelection},
            response_cls=ListValuesForComponentsResponse,
        )

    @classmethod
    def ungetValuesForComponents(cls, components: List[ComponentInfo], rootComponent: ComponentInfo, currentSelections: CurrentSelectionsMap) -> ServiceData:
        """
        The ungetValuesForComponents operation is used to undo the generated Smart Code for the Item ID if users do not
        want to proceed with the Item creation after generating the Smart Code. 
        
        For example after generating the Smart Code for Item ID, if users want to cancel Item creation,
        ungetValuesForComponents operation resets the Smart Code counters/numbers/attributes back to the earlier values
        so that these values are reusable
        """
        return cls.execute_soa_method(
            method_name='ungetValuesForComponents',
            library='Configuration',
            service_date='2010_04',
            service_name='SmartUiBldr',
            params={'components': components, 'rootComponent': rootComponent, 'currentSelections': currentSelections},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.Notification._2017_11.SubscriptionManagement import GetSubscribablePropertiesResponse
from typing import List
from tcsoa.base import TcService


class SubscriptionManagementService(TcService):

    @classmethod
    def getSubscribableProperties(cls, subscribableObjectTypes: List[BusinessObject]) -> GetSubscribablePropertiesResponse:
        """
        This operation returns a list of properties for the given subscribable object type to build subscription
        criteria for class based subscriptions.
        
        Use cases:
        This operation provides the following use case for Subscription:
        
        Use Case1: Display list of configured properties for subscribable object type to build subscription criteria
        for class based subscription.
        Administrator may configure subscription criteria properties for a subscribable object type using Subscription
        Administration Panel in RAC.
        - This operation returns list of configured properties if defined, for selected subscribable object type which
        can be used to build subscription criteria for class based subscription.
        - If the configuration is not defined by the admin for the selected subscribable object type than it will get
        configuration from its parent type up in the hierarchy until it finds one.
        - If configuration is not defined for the subscribable object type and all its parent type than default will
        list all properties for selected subscribable object type.
        
        """
        return cls.execute_soa_method(
            method_name='getSubscribableProperties',
            library='Internal-Notification',
            service_date='2017_11',
            service_name='SubscriptionManagement',
            params={'subscribableObjectTypes': subscribableObjectTypes},
            response_cls=GetSubscribablePropertiesResponse,
        )



from __future__ import annotations

from tcsoa.gen.Qualification._2014_06.QualificationManagement import QualificationLevelInfo, ManageQualificationInfo, RemoveUserQualificationInfo, UpdateQualificationInfo, AssignUserQualificationInfo
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class QualificationManagementService(TcService):

    @classmethod
    def removeQualificationLevel(cls, qualificationLevelInfo: List[QualificationLevelInfo]) -> ServiceData:
        """
        Removes a level from the list of Qualification levels. 
        """
        return cls.execute_soa_method(
            method_name='removeQualificationLevel',
            library='Qualification',
            service_date='2014_06',
            service_name='QualificationManagement',
            params={'qualificationLevelInfo': qualificationLevelInfo},
            response_cls=ServiceData,
        )

    @classmethod
    def removeUserQualification(cls, removeUserQualificationInfo: List[RemoveUserQualificationInfo]) -> ServiceData:
        """
        Removes a Fnd0Qualification object that is assigned to a Teamcenter User by deleting the Fnd0UserHasQual
        relation object that relates the User and the Fnd0Qualification.
        """
        return cls.execute_soa_method(
            method_name='removeUserQualification',
            library='Qualification',
            service_date='2014_06',
            service_name='QualificationManagement',
            params={'removeUserQualificationInfo': removeUserQualificationInfo},
            response_cls=ServiceData,
        )

    @classmethod
    def updateQualification(cls, updateQualificationInfo: List[UpdateQualificationInfo]) -> ServiceData:
        """
        This operation updates a list of Fnd0Qualification objects.
        """
        return cls.execute_soa_method(
            method_name='updateQualification',
            library='Qualification',
            service_date='2014_06',
            service_name='QualificationManagement',
            params={'updateQualificationInfo': updateQualificationInfo},
            response_cls=ServiceData,
        )

    @classmethod
    def createQualification(cls, qualificationInfo: List[ManageQualificationInfo]) -> ServiceData:
        """
        Creates a list of Fnd0Qualification objects. A single Fnd0Qualification object is created for each
        ManageQualificationInfo structure in the list.
        """
        return cls.execute_soa_method(
            method_name='createQualification',
            library='Qualification',
            service_date='2014_06',
            service_name='QualificationManagement',
            params={'qualificationInfo': qualificationInfo},
            response_cls=ServiceData,
        )

    @classmethod
    def appendQualificationLevel(cls, qualificationLevelInfo: List[QualificationLevelInfo]) -> ServiceData:
        """
        Appends a level to the ordered list of Fnd0Qualification levels.
        """
        return cls.execute_soa_method(
            method_name='appendQualificationLevel',
            library='Qualification',
            service_date='2014_06',
            service_name='QualificationManagement',
            params={'qualificationLevelInfo': qualificationLevelInfo},
            response_cls=ServiceData,
        )

    @classmethod
    def assignUserQualification(cls, assignUserQualificationInfo: List[AssignUserQualificationInfo]) -> ServiceData:
        """
        Assigns a Fnd0Qualification object to a Teamcenter User. Each AssignUserQualificationInfo structure in the
        input list contains information required to assign a Fnd0Qualification object to a Teamcenter User. A
        Fnd0UserHasQual relation object will be created on successful assignment of Fnd0Qualification to a User.
        """
        return cls.execute_soa_method(
            method_name='assignUserQualification',
            library='Qualification',
            service_date='2014_06',
            service_name='QualificationManagement',
            params={'assignUserQualificationInfo': assignUserQualificationInfo},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Core._2012_02.DataManagement import BulkCreIn, WhereUsedResponse, WhereUsedInputData, ValidationResponse, WhereUsedConfigParameters
from tcsoa.gen.Core._2008_06.DataManagement import CreateResponse, CreateIn
from tcsoa.gen.Core._2012_02.OperationDescriptor import DeepCopyDataInput, GetDeepCopyDataResponse
from typing import List
from tcsoa.gen.Core._2012_02.Session import SetPolicyResponse, RegisterIndex
from tcsoa.base import TcService


class SessionService(TcService):

    @classmethod
    def registerState(cls, level: str) -> RegisterIndex:
        """
        Register desired level for server state as used by the Server Manager to control server timeout. Note that an
        attempt to register at LEVEL_STATELESS is ignored since this is the default level when no registrations are in
        effect. To move from a higher level to the stateless level the 'unregister' operation should be used to delete
        the earlier registration. Note that it is possible to be registered at more than one level and there may be
        multiple registrations at each level.
        """
        return cls.execute_soa_method(
            method_name='registerState',
            library='Core',
            service_date='2012_02',
            service_name='Session',
            params={'level': level},
            response_cls=RegisterIndex,
        )

    @classmethod
    def setObjectPropertyPolicy(cls, policyName: str, useRefCounting: bool) -> SetPolicyResponse:
        """
        Sets the current object property policy. The business logic of a service operation determines what business
        objects are returned, while the object property policy determines which properties are returned on each
        business object instance. This allows the client application to determine how much or how little data is
        returned based on how the client application uses those returned business objects. The policy is applied
        uniformly to all service operations. 
        By default, all applications use the Default object property policy, defined on the Teamcenter server
        '$TC_DATA/soa/policies/default.xml. 'It is this policy that is applied to all service operation responses until
        the client application changes the policy. Siemens PLM Software strongly recommends that all applications
        change the policy to one applicable to the client early in the session.
        The object property policy is set to the policy named in the file '$TC_DATA/soa/policies/<policyName>.xml' The
        reserved policy name "Empty", will enforce a policy that only returns minimum data required for each object
        (UID and type name).The object property policy will stay in affect for this session until changed by another
        call to 'setObjectPropertyPolicy'.  The full policy is returned where the client application can modify it
        throughout the session via calls to 'updatObjectPropertyPolicy'.
        
        Like any other service operation, this operation cannot be called before establishing a session with the
        'login' serivce operation, so if you need a policy other than the Default policy for the business objects
        returned by the login operation, use the _2011_06 version of the 'login/loginSso' operation to authenticate and
        establish a session without returning business objects. The 'setObjectPropertyPolicy' operation can then be
        called to establish the policy for the session.
        
        
        Exceptions:
        >If the named policy does not exist or there are errors parsing the XML file (error code 214104).
        """
        return cls.execute_soa_method(
            method_name='setObjectPropertyPolicy',
            library='Core',
            service_date='2012_02',
            service_name='Session',
            params={'policyName': policyName, 'useRefCounting': useRefCounting},
            response_cls=SetPolicyResponse,
        )

    @classmethod
    def unregisterState(cls, index: int) -> bool:
        """
        Remove the specified registration.
        """
        return cls.execute_soa_method(
            method_name='unregisterState',
            library='Core',
            service_date='2012_02',
            service_name='Session',
            params={'index': index},
            response_cls=bool,
        )


class DataManagementService(TcService):

    @classmethod
    def validateIdValue(cls, input: List[CreateIn]) -> ValidationResponse:
        """
        This operation determines if the given 'MultiFieldKey' properties are unique based on the 'MultiFieldKey'
        definition.
        
        Use cases:
        Use this operation before creating a new object to validate if the 'MultiFieldKey' property combination is
        already used.  This is an existence check rather than a true validation. It does not validate the property
        values against Naming Rules.
        """
        return cls.execute_soa_method(
            method_name='validateIdValue',
            library='Core',
            service_date='2012_02',
            service_name='DataManagement',
            params={'input': input},
            response_cls=ValidationResponse,
        )

    @classmethod
    def whereUsed(cls, input: List[WhereUsedInputData], configParams: WhereUsedConfigParameters) -> WhereUsedResponse:
        """
        The 'whereUsed' service identifies all the parent Item and ItemRevision objects in the structure where the
        input Item or ItemRevision is used. User can provide RevisionRule to search for specific ItemRevision . By
        default all ItemRevision objects are returned. The number of levels of 'whereUsed' search indicates, whether to
        return one or top or all levels of assemblies. It supports 'whereUsed' search on any WorkspaceObject which
        implements the "'whereUsed'" interface.
        
        Use cases:
        A user performs 'whereUsed' search to find all the assemblies that contain a particular Item or ItemRevision.
        User inputs Item or ItemRevision and the search can be made with following options:
        - RevisionRule  This can be set to All, displaying all ItemRevision objects  that have an occurrence of target
        ItemRevision. If a specific RevisionRule is selected only the ItemRevision objects  configured by the rule are
        returned in the search.
        - Depth up to which numbers of levels are to be returned.
        - Boolean representing whether to only send back precise parents (used by ItemRevision specifically and not
        Item)
        
        
        
        Additional Configuration Parameters can be used to do customized whereUsed search. 'WhereUsedConfigParameters'
        has below maps:
        
        stringMap ( std::string, std::string ) 
        doubleMap ( std::string, double ) 
        intMap ( std::string, int ) 
        boolMap ( std::string, bool ) 
        dateMap ( std::string, Teamcenter::DateTime ) 
        tagMap ( std::string, BusinessObjectRef ( Teamcenter::BusinessObject ) ) 
        floatMap ( std::string, float ) 
        
        COTS whereUsed search uses 'tagMap' to specify the RevisionRule with revision_rule as a key, 'boolMap' to
        specify whereUsedPreciseFlag with whereUsedPreciseFlag as a key, and 'intMap' to specify number of levels with
        numLevels as a key. Similarly other maps can be used to pass additional parameters for customized whereUsed
        search. Teamcenter currently don't have any extension where additional Configuration Parameters can be
        considered for 'whereUsed' search if passed by user. User can implement "whereUsed" interface on any custom
        WorkspaceObject to consider additional Configuration Parameters for 'whereUsed' search.
        
        The output contains list of  each parent level search result in the structure.
        """
        return cls.execute_soa_method(
            method_name='whereUsed',
            library='Core',
            service_date='2012_02',
            service_name='DataManagement',
            params={'input': input, 'configParams': configParams},
            response_cls=WhereUsedResponse,
        )

    @classmethod
    def bulkCreateObjects(cls, input: List[BulkCreIn]) -> CreateResponse:
        """
        This is a generic operation for bulk creation of Business Objects. This will create instances of the given
        quantity of the specified type in their specified containing folders. This will also create any
        secondary(compounded) objects that need to be created, assuming the CreateInput for the secondary object is
        represented in the recursive CreateInput object e.g. Item is Primary Object that also creates Item Master,
        ItemRevision and ItemRevision in turn creates ItemRevision Master. The input for all these levels is passed in
        through the recursive CreateInput object. This operation is applicable for bulk creation of Item, Form ,Dataset
        and Asp0Aspect Types only.
        
        Use cases:
        1. To create large number of objects of specified types namely Item, Dataset and Form each of specified
        quantities and save them through a single transaction to significantly reduce the number of sql queries
        incurred during object creation, thus improving object creation performance and making object creation scalable.
        2. To create large number of Asp0Aspect objects of specified types and save them through a single transaction
        to significantly reduce the number of sql queries incurred during object creation, thus improving object
        creation performance and making object creation scalable.
        """
        return cls.execute_soa_method(
            method_name='bulkCreateObjects',
            library='Core',
            service_date='2012_02',
            service_name='DataManagement',
            params={'input': input},
            response_cls=CreateResponse,
        )


class OperationDescriptorService(TcService):

    @classmethod
    def getDeepCopyData(cls, deepCopyDataInput: List[DeepCopyDataInput]) -> GetDeepCopyDataResponse:
        """
        This operation returns information required to perform SaveAs on a Business Object instance.
        
        Use cases:
        Client constructs the SaveAs dialog for a Business Object using this operation. The information returned by
        this operation allows a client to construct the SaveAs dialogs and the DeepCopy panels for user input. Once the
        user input is received, client can make subsequent invocation to the 'DataManagement.saveAsObjects' operation
        to execute SaveAs on the object.
        """
        return cls.execute_soa_method(
            method_name='getDeepCopyData',
            library='Core',
            service_date='2012_02',
            service_name='OperationDescriptor',
            params={'deepCopyDataInput': deepCopyDataInput},
            response_cls=GetDeepCopyDataResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.ActiveWorkspaceBom._2019_12.OccurrenceConfiguration import ConfigRuleResponse2, CreateOrUpdateClassicVariantRuleData, ClassicVariantsResp, ClassicVariantsData
from tcsoa.gen.Internal.ActiveWorkspaceBom._2017_06.OccurrenceConfiguration import ConfigRuleInput
from tcsoa.gen.Internal.ActiveWorkspaceBom._2019_12.OccurrenceManagement import FindMatchingFilterResponse2, RemoveInContextOverrides, SubsetResponse2, OccurrencesData3, DetachObjectsInputData, FindMatchingFilterInput2, InfoForAddElemData3, OccurrencesResp3
from tcsoa.gen.Internal.ActiveWorkspaceBom._2019_12.DataManagement import GetViewModelForCreateInfo, GetViewModelForCreateResponse
from tcsoa.gen.Internal.ActiveWorkspaceBom._2012_10.OccurrenceManagement import InfoForAddElemResp
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.Internal.ActiveWorkspaceBom._2016_03.OccurrenceManagement import SubsetInput2
from tcsoa.base import TcService


class OccurrenceConfigurationService(TcService):

    @classmethod
    def createOrUpdateClassicVariantRule(cls, inputData: CreateOrUpdateClassicVariantRuleData) -> ClassicVariantsResp:
        """
        This operation sets options and values provided in the input for a given product context. It returns a
        VariantRule and list of option and associated list of value. When input contains VariantRule then the list of
        options and values provided in the input are merged with this input VariantRule.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateClassicVariantRule',
            library='Internal-ActiveWorkspaceBom',
            service_date='2019_12',
            service_name='OccurrenceConfiguration',
            params={'inputData': inputData},
            response_cls=ClassicVariantsResp,
        )

    @classmethod
    def getClassicVariants(cls, inputData: ClassicVariantsData) -> ClassicVariantsResp:
        """
        This operation returns list of option and associated list of value for given product context. When input
        contains VariantRule then the list of options and values are returned along with how set indication.
        """
        return cls.execute_soa_method(
            method_name='getClassicVariants',
            library='Internal-ActiveWorkspaceBom',
            service_date='2019_12',
            service_name='OccurrenceConfiguration',
            params={'inputData': inputData},
            response_cls=ClassicVariantsResp,
        )

    @classmethod
    def getConfigurationRules2(cls, input: ConfigRuleInput) -> ConfigRuleResponse2:
        """
        This operation returns a list of RevisionRule or VariantRule based on the input product context information.
        The number of RevisionRule or VariantRule in the response depends on the page size.
        """
        return cls.execute_soa_method(
            method_name='getConfigurationRules2',
            library='Internal-ActiveWorkspaceBom',
            service_date='2019_12',
            service_name='OccurrenceConfiguration',
            params={'input': input},
            response_cls=ConfigRuleResponse2,
        )


class OccurrenceManagementService(TcService):

    @classmethod
    def detachObjects(cls, input: List[DetachObjectsInputData]) -> ServiceData:
        """
        This operation detaches the secondary objects from the primary object.
        """
        return cls.execute_soa_method(
            method_name='detachObjects',
            library='Internal-ActiveWorkspaceBom',
            service_date='2019_12',
            service_name='OccurrenceManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def findMatchingFilters2(cls, input: List[FindMatchingFilterInput2]) -> FindMatchingFilterResponse2:
        """
        This operation retrieves filters matching input search string.
        """
        return cls.execute_soa_method(
            method_name='findMatchingFilters2',
            library='Internal-ActiveWorkspaceBom',
            service_date='2019_12',
            service_name='OccurrenceManagement',
            params={'input': input},
            response_cls=FindMatchingFilterResponse2,
        )

    @classmethod
    def getInfoForAddElement3(cls, getInfoForElementIn: InfoForAddElemData3) -> InfoForAddElemResp:
        """
        This operation retrieves information required for creating an Awb0Element under product specified in
        Awb0Element. Operation returns allowed child occurrence revision type name(s) for given parent type when
        "fetchAllowedOccRevTypes" is set to true. Otherwise operation returns allowed Item type name(s) for given
        parentElement. The operation also returns preferred type as parent type if that is specified as allowed child
        type in "TCAllowedChildTypes_<ParentItemType>" preference; otherwise preferred type is empty.
        """
        return cls.execute_soa_method(
            method_name='getInfoForAddElement3',
            library='Internal-ActiveWorkspaceBom',
            service_date='2019_12',
            service_name='OccurrenceManagement',
            params={'getInfoForElementIn': getInfoForElementIn},
            response_cls=InfoForAddElemResp,
        )

    @classmethod
    def getOccurrences3(cls, inputData: OccurrencesData3) -> OccurrencesResp3:
        """
        Retrieves the page of configured occurrences for given the top-level product and configuration parameters as
        input. The service provides the facility to optionally filter and sort the result by additional filters and
        sorting criteria that may be provided as input. The output also contains a cursor that defines the place to
        start the next page location. The cursor must be passed back in to any subsequent call to get the next page of
        occurrences.
        
        Exceptions:
        >Teamcenter::Soa::Server::ServiceException
        """
        return cls.execute_soa_method(
            method_name='getOccurrences3',
            library='Internal-ActiveWorkspaceBom',
            service_date='2019_12',
            service_name='OccurrenceManagement',
            params={'inputData': inputData},
            response_cls=OccurrencesResp3,
        )

    @classmethod
    def getSubsetInfo3(cls, subsetInputs: List[SubsetInput2]) -> SubsetResponse2:
        """
        This operation retrieves filters and recipes which are used to find matching Awb0Element objects for input
        Awb0ProductContextInfo.
        """
        return cls.execute_soa_method(
            method_name='getSubsetInfo3',
            library='Internal-ActiveWorkspaceBom',
            service_date='2019_12',
            service_name='OccurrenceManagement',
            params={'subsetInputs': subsetInputs},
            response_cls=SubsetResponse2,
        )

    @classmethod
    def removeInContextPropertyOverride(cls, removeInContextOverridesInfo: RemoveInContextOverrides) -> ServiceData:
        """
        The operation removes overridden property for the Awb0Element .
        
        Exceptions:
        >This operation may raise a ServiceException containing following 
        errors:
        
        126001 -  An internal error has occurred in the Occurrence Management Module.
        43021 -  The BOM View Revision cannot be modified. Please check your access rights.
        """
        return cls.execute_soa_method(
            method_name='removeInContextPropertyOverride',
            library='Internal-ActiveWorkspaceBom',
            service_date='2019_12',
            service_name='OccurrenceManagement',
            params={'removeInContextOverridesInfo': removeInContextOverridesInfo},
            response_cls=ServiceData,
        )


class DataManagementService(TcService):

    @classmethod
    def getViewModelForCreate(cls, input: GetViewModelForCreateInfo) -> GetViewModelForCreateResponse:
        """
        This operation retrieves view model definition to author WorkspaceObject. The response contains the definition
        of View Model information based on CreateInput object and client derives data to render the editable row UI
        element/widget to create the object. The editable row represents the ViewModelProperty for the CreateInput
        object. The editable cells represent the ViewModelProperty for the properties on CreateInput, cell value can
        have auto-assigned initial property value if any. This is like 'Add Panel' scenario, i.e. panel is based on
        'Form' view and this operation provides definition for 'Grid:Row' view display.
        
        Use cases:
        Use case 1: Create child under selected object.
        When user selects an object in Active Workspace to add new child, the operation getViewModelForCreate retrieves
        view model information to render widget to create an object.
        
        Use case 2: Create Sibling for a selected object.
        When user selects an object in Active Workspace to add new sibling, the operation getViewModelForCreate
        retrieves view model information to render widget to create an object.
        """
        return cls.execute_soa_method(
            method_name='getViewModelForCreate',
            library='Internal-ActiveWorkspaceBom',
            service_date='2019_12',
            service_name='DataManagement',
            params={'input': input},
            response_cls=GetViewModelForCreateResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, RevisionRule
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindAndFixInput(TcBaseObj):
    """
    This structure provides a set of input values for searching candidates for broken links.
    
    :var brokenLinkBOMLines: The broken links or the scope of the broken links.
    :var quickSearch: Do quick search for broken links and candidates?
    :var criteria: The criteria used to search candidates.
    :var autofixAllowed: Automatically fix the broken links if one-to-one match is found?
    """
    brokenLinkBOMLines: List[BOMLine] = ()
    quickSearch: bool = False
    criteria: SearchCandidatesCriteria = None
    autofixAllowed: bool = False


@dataclass
class FindCandidatesResponse(TcBaseObj):
    """
    This structure provides the return of the operation.
    
    :var foundCandidates: The candidates found.
    :var serviceData: The service data to be returned.
    """
    foundCandidates: List[CandidatesList] = ()
    serviceData: ServiceData = None


@dataclass
class RepairBrokenLinksParam(TcBaseObj):
    """
    This structure provides a set of broken link and candidate pair.
    
    :var brokenLink: The broken link to fix.
    :var candidate: The candidate for the broken link.
    """
    brokenLink: BOMLine = None
    candidate: BOMLine = None


@dataclass
class SearchCandidatesCriteria(TcBaseObj):
    """
    This structure provides a set of criteria for searching candidates for broken links including
    candidate scope and previous configuration.
    
    :var candidateScopeBOMLines: the selected BOM lines from where to search the candidates
    :var criteria: the criteria to match BOM lines and the broken link
    :var previousConfiguration: the configuration of the structure to retrieve the original
    value of the abs occ properties of the broken link
    """
    candidateScopeBOMLines: List[BOMLine] = ()
    criteria: List[CriteriaStruct] = ()
    previousConfiguration: RevisionRule = None


@dataclass
class CandidatesList(TcBaseObj):
    """
    This structure provides a set of candidates for a broken link.
    
    :var brokenLink: the BOM line of the broken link
    :var candidates: the candidates of the broken link. When empty means no candidate or
    no search for candidates performed
    """
    brokenLink: BOMLine = None
    candidates: List[BOMLine] = ()


@dataclass
class CriteriaStruct(TcBaseObj):
    """
    This structure provides a property clause of criteria for searching candidates for broken links.
    
    :var propertyName: propertyName
    :var operatorName: operatorName
    """
    propertyName: str = ''
    operatorName: str = ''



from __future__ import annotations

from tcsoa.gen.Manufacturing._2014_12.StructureSearch import AdditionalInfo
from tcsoa.gen.BusinessObjects import BOMLine
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class PartsInProximityOutput(TcBaseObj):
    """
    This structure contains the following information:
    - proximityDistance - The proximity distance (in mm) for which the operation was carried out.
    - featureToPartsInProximity - A map (BOMLine, list of BOMLine) containing key as the BOMLine object of type
    Mfg0BvrManufacturingFeature and value as its corresponding list of BOMLine objects that represent parts which are
    in proximity of given proximity distance.
    - additionalInfoOut - Any additional output information. Reserved for future use.
    
    
    
    :var proximityDistance: The proximity distance (in mm) for which the operation was carried out.
    :var featureToPartsInProximity: A map (BOMLine, list of BOMLine) containing key as the BOMLine object of type
    Mfg0BvrManufacturingFeature and value as its corresponding list of BOMLine objects that represent parts which are
    in proximity of given proximity distance.
    :var additionalInfoOut: Any additional output information. Reserved for future use.
    """
    proximityDistance: float = 0.0
    featureToPartsInProximity: MFGFeatureToPartsMap = None
    additionalInfoOut: AdditionalInfo = None


@dataclass
class PartsInProximityResponse(TcBaseObj):
    """
    The response of the service operation contains the following:
    - partsInProximityOut - A list containing returned information after processing of the operation. This will contain
    the proximity distance, a map of BOMLine objects of type Mfg0BvrManufacturingFeature and their corresponding list
    of BOMLine objects representing parts within the proximity distance criteria provided.
    - serviceData - Partial errors as part of the serviceData.
    
    
    
    :var partsInProximityOut: A list containing returned information after processing of the operation. This will
    contain the proximity distance, a map of BOMLine objects of type Mfg0BvrManufacturingFeature and their
    corresponding list of BOMLine objects representing parts within the proximity distance criteria provided.
    :var serviceData: Partial errors as part of the serviceData.
    """
    partsInProximityOut: List[PartsInProximityOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ProximityCriteriaInput(TcBaseObj):
    """
    This structure contains the following information:
    - proximityDistance - The distance in "mm", which represent the proximity distance from BOMLine of type
    Mfg0BvrManufacturingFeature to search for closest BOMLine objects representing parts.
    - mfgFeatureLines - A list of BOMLine objects of type Mfg0BvrManufacturingFeature for which to find the parts in
    proximity.
    - additionalInfoIn - Any additional input information. Reserved for future use.
    
    
    
    :var proximityDistance: The distance in "mm", which represent the proximity distance from BOMLine of type
    Mfg0BvrManufacturingFeature to search for closest BOMLine objects representing parts.
    :var mfgFeatureLines: A list of BOMLine objects of type Mfg0BvrManufacturingFeature for which to find the parts in
    proximity.
    :var additionalInfoIn: Any additional input information. Reserved for future use.
    """
    proximityDistance: float = 0.0
    mfgFeatureLines: List[BOMLine] = ()
    additionalInfoIn: AdditionalInfo = None


"""
A Map (BOMLine,  list of BOMLine) containing key as the BOMLine object of type Mfg0BvrManufacturingFeature and value as its corresponding list of BOMLine objects that represent parts which are in proximity of given proximity distance.
"""
MFGFeatureToPartsMap = Dict[BOMLine, List[BOMLine]]



from __future__ import annotations

from tcsoa.gen.Core._2020_01.ProjectLevelSecurity import ProjectTeamPagedResponse, UserProjectsAndPrivilegeResponse, ChildStructureResponse, GroupRoleNode, SetPrivilegeForUserInput, ProjectTeamPagedInput, ProjectPrivilegeResponse, AddOrRemoveProjectMemberInput
from tcsoa.gen.Core._2020_01.DataManagement import IDContextOutput, IdentifierTypesOut, IdentifierTypesIn, IDDispRuleCreateIn
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.BusinessObjects import WorkspaceObject, TC_Project
from tcsoa.base import TcService


class ProjectLevelSecurityService(TcService):

    @classmethod
    def setUserPrivilege(cls, inputs: List[SetPrivilegeForUserInput]) -> ServiceData:
        """
        Set the privilege of the given User objects in ProjectTeam of the given TC_Project.
        
        Use cases:
        This service allows the project team administrator to achieve the following use cases.
        
        Use Case 1: The project team administrator sets a list of User objects to be non-privileged in the ProjectTeam
        of a TC_Project.
        Use Case 2: The project team administrator sets a list of User objects to be privileged in the ProjectTeam of a
        TC_Project.
        Use Case 3: The project team administrator sets a list of User objects to be project team administrator in the
        ProjectTeam of a TC_Project.
        """
        return cls.execute_soa_method(
            method_name='setUserPrivilege',
            library='Core',
            service_date='2020_01',
            service_name='ProjectLevelSecurity',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )

    @classmethod
    def addOrRemoveProjectMembers(cls, inputs: List[AddOrRemoveProjectMemberInput]) -> ServiceData:
        """
        Adds or removes GroupMember objects or Group objects to/from the ProjectTeam of the given TC_Project.
        
        Use cases:
        Use Case 1: The login user selects a TC_Project and adds some Group or GroupMember objects to the ProjectTeam.
        Use Case 2: The login user selects a TC_Project and removes some Group or GroupMember objects from the
        ProjectTeam.
        """
        return cls.execute_soa_method(
            method_name='addOrRemoveProjectMembers',
            library='Core',
            service_date='2020_01',
            service_name='ProjectLevelSecurity',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )

    @classmethod
    def getFirstLevelProjectTeamStructure(cls, input: ProjectTeamPagedInput) -> ProjectTeamPagedResponse:
        """
        This operations returns the paginated output of the first level nodes of the ProjectTeam for the given
        TC_Project object.
        
        Use cases:
        Select a TC_Project object to display all first level nodes in the ProjectTeam tree.
        """
        return cls.execute_soa_method(
            method_name='getFirstLevelProjectTeamStructure',
            library='Core',
            service_date='2020_01',
            service_name='ProjectLevelSecurity',
            params={'input': input},
            response_cls=ProjectTeamPagedResponse,
        )

    @classmethod
    def getModifiableProjects(cls, startIndex: int, pageSize: int) -> UserProjectsAndPrivilegeResponse:
        """
        This operation returns the TC_Project objects that the login user can modify. The TC_Project objects in the
        response are based on the pagination input.
        
        Use cases:
        A User loads all modifiable project.
        """
        return cls.execute_soa_method(
            method_name='getModifiableProjects',
            library='Core',
            service_date='2020_01',
            service_name='ProjectLevelSecurity',
            params={'startIndex': startIndex, 'pageSize': pageSize},
            response_cls=UserProjectsAndPrivilegeResponse,
        )

    @classmethod
    def getPrivilegeInProjects(cls, projects: List[TC_Project]) -> ProjectPrivilegeResponse:
        """
        This operation returns the privilege of the current user in each TC_Project object in the input.
        
        Use cases:
        When the login user checks the privilege in a list of TC_Project objects, the operation returns the privilege
        value of the login user of each project.
        """
        return cls.execute_soa_method(
            method_name='getPrivilegeInProjects',
            library='Core',
            service_date='2020_01',
            service_name='ProjectLevelSecurity',
            params={'projects': projects},
            response_cls=ProjectPrivilegeResponse,
        )

    @classmethod
    def getProjectTeamChildNodes(cls, project: TC_Project, node: GroupRoleNode, depth: int) -> ChildStructureResponse:
        """
        The operation returns child nodes for the given Group node or a Role node in the ProjectTeam tree based on the
        given depth.
        
        Use cases:
        Expanding a Group or a Role node in a Project Team tree, the child nodes are returned.
        """
        return cls.execute_soa_method(
            method_name='getProjectTeamChildNodes',
            library='Core',
            service_date='2020_01',
            service_name='ProjectLevelSecurity',
            params={'project': project, 'node': node, 'depth': depth},
            response_cls=ChildStructureResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def createIdDisplayRules(cls, idDispRuleCreIns: List[IDDispRuleCreateIn]) -> ServiceData:
        """
        This operation creates the ID Display Rules (IdDispRule) with the input ID Context information.
        ID Display Rule contains the list of ID Contexts and their order. Based on the order of the ID Contexts
        defined, the system evaluates the display name of the Item and ItemRevision from their Alternate IDs.
        
        ID Context (IdContext), represents the context information under which the OEM defines the unique IDs of their
        Item and ItemRevision. This context information is used when Teamcenter users define the unique IDs of Item and
        ItemRevision objects.
        
        User can set one of the ID Display Rules as the current ID Display Rule. The current ID Display Rule is used to
        evaluate the display names of the Item and ItemRevision. In case the ID Context of the Alternate ID with the
        Item and ItemRevision object does not match with that of the current ID Display Rule then system uses the
        default ID Display Rule to evaluate the display names of Item and ItemRevision objects.
        """
        return cls.execute_soa_method(
            method_name='createIdDisplayRules',
            library='Core',
            service_date='2020_01',
            service_name='DataManagement',
            params={'idDispRuleCreIns': idDispRuleCreIns},
            response_cls=ServiceData,
        )

    @classmethod
    def getIdContexts(cls, inputObjs: List[WorkspaceObject]) -> IDContextOutput:
        """
        This operation fetches all instances of the ID Context objects (IdContext) from the Teamcenter database
        applicable for the input objects of type Item and ItemRevision based on defined ID Context Rules
        (IdContextRule) by the system administrators.
        
        This operation queries ID Context Rule objects and fetches the ID Context objects based on the input Item,
        ItemRevision or null. The input is the identifiable type defined on the ID Context Rules. For a null input, it
        returns the Id Context objects where the identifiable type is null.
        All ID Context objects from the Teamcenter data base are returned in case input object is other than Item,
        ItemRevision or null. An empty input list would also return all ID Context objects from the Teamcenter data
        base.
        
        IdContext objects represents the context information under which the OEM defines the unique IDs of their Item
        and ItemRevision objects. This context information is used when Teamcenter users define the unique IDs of Item
        and ItemRevision objects.
        
        Alternate and Alias IDs of Teamcenter are the example of the such unique IDs of Item and ItemRevision. Users
        define Alternate and Alias IDs with the help of the ID Context as one of the unique attribute of the ID.
        """
        return cls.execute_soa_method(
            method_name='getIdContexts',
            library='Core',
            service_date='2020_01',
            service_name='DataManagement',
            params={'inputObjs': inputObjs},
            response_cls=IDContextOutput,
        )

    @classmethod
    def getIdentifierTypes(cls, identifierTypesIn: List[IdentifierTypesIn]) -> IdentifierTypesOut:
        """
        This operation fetches the applicable Identifier types for the input objects of type Item and/or ItemRevision
        along with the input IdContext object. System queries the ID Context Rules defined in Teamcenter database and
        retrives the Identifier types.
        
        Alternate and Alias IDs are defined in Teamcenter as instances of business object of type Identifier. ID
        Context, of business object type IdContext, represents the context information under which the OEM defines the
        unique IDs of their Item and ItemRevision. This context information is used when Teamcenter users define the
        Alternate and Alias IDs of Item and ItemRevision objects.
        
        ID Context Rules are defined as instances of business object type IdContextRule in Teamcenter database. These
        rules map the combination of ID Context and the object type e.g.  Item or ItemRevision, called Identifiable
        types, to the type of the Identifier applicable in the context.
        
        This operation also returns the other applicable objects for which Alternate IDs along with the input objects
        can be defined. In case of input objects of type Item, this operation returns the list of revision objects of
        the Item, and in case of input objects of type ItemRevision, this operation returns the Item object as the
        applicable object for which Alternate IDs can be defined.
        """
        return cls.execute_soa_method(
            method_name='getIdentifierTypes',
            library='Core',
            service_date='2020_01',
            service_name='DataManagement',
            params={'identifierTypesIn': identifierTypesIn},
            response_cls=IdentifierTypesOut,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GmsSyncStatus(TcBaseObj):
    """
    It holds input object in GSIdentity format and its sync status.
    
    :var candidateObject: Input GSIdentity string.
    :var syncStatus: The status of object which can be one of the following
    1.    OUT OF DATE
    2.    UP TO DATE
    3.    UNDETERMINED
    """
    candidateObject: str = ''
    syncStatus: str = ''


@dataclass
class ObjectsByClass(TcBaseObj):
    """
    It holds the class and list of objects belonging to that class which needs to be checked for modification.
    
    :var className: Class name for which modified objects are returned.
    :var objectsList: Optional list of objects(string representation of BusinessObject) corresponding to class  which
    need to be checked for modification.
    """
    className: str = ''
    objectsList: List[str] = ()


@dataclass
class OwnershipChangeReplicaUpdateResponse(TcBaseObj):
    """
    It holds FMS ticket of log file and errors if any.
    
    :var logFmsTicket: The FMS ticket of log file which has status of operation either success or failure.
    :var serviceData: The service data
    """
    logFmsTicket: str = ''
    serviceData: ServiceData = None


@dataclass
class ReplicaDeletionMasterUpdateResponse(TcBaseObj):
    """
    It holds the FMS ticket of log file which has status of an operation and errors if any.
    
    :var logFmsTicket: The FMS ticket of log file which has status of operation either success or failure.
    :var serviceData: The service data
    """
    logFmsTicket: str = ''
    serviceData: ServiceData = None


@dataclass
class StubReplicationMasterUpdateResponse(TcBaseObj):
    """
    It holds FMS ticket to log file which has success or failure information for each input GSIdentity and errors if
    any.
    
    :var logFmsTicket: FMS ticket to log file
    :var serviceData: The service data
    """
    logFmsTicket: str = ''
    serviceData: ServiceData = None


@dataclass
class SyncResponse(TcBaseObj):
    """
    It holds the objects that were modified since last export and errors if any.
    
    :var candidateObjectsList: A list of objects(string representation of BusinessObject) which were modified since
    last export or need synchronization.
    :var serviceData: The service data
    """
    candidateObjectsList: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class CheckSyncStateResponse(TcBaseObj):
    """
    It holds a list replica sync status for each input object, FMS ticket of log file and errors if any.
    
    :var replicaSyncStatusList: A list of GSIdentity and it's sync status
    :var logFmsTicket: The FMS ticket of log file which has status for each object with is either success or failed.
    
    :var serviceData: The service data
    """
    replicaSyncStatusList: List[GmsSyncStatus] = ()
    logFmsTicket: str = ''
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, RevisionRule
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ReplacementCandidate(TcBaseObj):
    """
    Structure containing the list of replacement candidates and the coresponding properties list for which the match is
    found.
    
    :var candidate: Candidate found on the basis of given search criteria
    :var matchedProps: The list of property on candidate matching the search criteria
    """
    candidate: BusinessObject = None
    matchedProps: List[str] = ()


@dataclass
class ReplacementCandidateSearchCriteria(TcBaseObj):
    """
    It represents property clause of criteria for searching candidates for broken links.
    
    :var operatorName: The type of search to perform "Mandatory" or "Optional".
    :var propertyName: The name of BOMLine property.
    :var propertyValue: The value of property
    """
    operatorName: str = ''
    propertyName: str = ''
    propertyValue: str = ''


@dataclass
class ReplacementCandidateSearchInput(TcBaseObj):
    """
    Represents the scope and search criteria to find the replacement candidates for broken links.
    
    :var searchScopes: The scope is a BOMLine to search  for the replacements candidates for the given broken link.
    :var replacementCandidateSearchCriteria: The list of search criteria to find the replacement candidate within the
    scope.
    :var revisionRule: The configuration of the structure to retrieve the original value of the abs occ properties of
    the broken link.
    """
    searchScopes: List[BusinessObject] = ()
    replacementCandidateSearchCriteria: List[ReplacementCandidateSearchCriteria] = ()
    revisionRule: RevisionRule = None


@dataclass
class BrokenLinkReplacementResponse(TcBaseObj):
    """
    The response contains service data and list of broken links, and the replacement candidates as per the given search
    criteria  
    
    :var brokenLinkWithReplacementCandidates: The list of broken links and replacement candidates.
    :var serviceData: The service data containing partial errors if any.
    """
    brokenLinkWithReplacementCandidates: List[BrokenLinkWithReplacementCandidate] = ()
    serviceData: ServiceData = None


@dataclass
class BrokenLinkSearchInput(TcBaseObj):
    """
    Input to the service is a structure containing the scope from BOP to find the broken links, scope from product
    structure to find the candidates and the search criteria .
    
    :var searchScope: The scope is a BOMLine to  search  for the broken links.
    :var replacementCandidateSearchInput: List of search criterion to find the replacement candidates for the given
    broken links.
    :var autoRepair: If true the broken link is automatically resolved if the single replacement candidate is found.
    :var fullStructure: If true the search is to be performed in full structure or only in visible structure.
    """
    searchScope: List[BusinessObject] = ()
    replacementCandidateSearchInput: ReplacementCandidateSearchInput = None
    autoRepair: bool = False
    fullStructure: bool = False


@dataclass
class BrokenLinkWithReplacementCandidate(TcBaseObj):
    """
    It represents the list of broken link and its replacement candidates
    
    :var brokenLink: The BOMLine found as a broken link 
    :var replacementCandidates: The list replacement candidates that are found as per the criteria
    """
    brokenLink: BusinessObject = None
    replacementCandidates: List[ReplacementCandidate] = ()



from __future__ import annotations

from tcsoa.gen.Diagramming._2014_06.DNDManagement import CreateConnectionPortsAndConnectResponse, CreateAndPasteResponse, CreateAndPasteInputInfo, CreateConnectionPortsAndConnectInputInfo
from typing import List
from tcsoa.base import TcService


class DNDManagementService(TcService):

    @classmethod
    def createAndPaste(cls, inputData: List[CreateAndPasteInputInfo]) -> CreateAndPasteResponse:
        """
        This operation creates an object of given type and pastes the newly created object onto the paste target
        object.The result of the operation is newly created object and any runtime object as a result of paste
        operation.
        
        Use cases:
        Use Case1: 
        Create a new object and paste it under the paste target. User drags and drops shapes onto the diagram. Shapes
        are mapped to specific types and a mapping file specifies this information. On drag and drop a new object of
        the desired type is created and is pasted on to the diagram root object. When the diagram is for a BOM
        structure the paste operation also returns a resultant runtime object [BOMLine] in addition to the newly
        created object. The runtime/newly created object is mapped to the shape on the diagram.
        
        Use Case2: 
        Connection Management
        
        Use Case2a:
        Create Connection, Ports and establish connection between them. User drags and drops a Connection shape onto
        the diagram and glues the same to Block shapes. Two port objects are created and pasted under the Block objects
        if connected_ToRules is set to allow connection between ports; a Connection object is created and pasted under
        its paste target object. Connection is established between the Connection object and Port objects.
        
        Use Case2b: 
        Create Connection, establish connection between input ports. User drags and drops a Connection shape onto the
        diagram and glues the same to Port shapes. A Connection object is created and pasted under the paste target.
        Connection is established between the Connection object and Port objects if connected_ToRules is set to allow
        connection between ports.
        
        Use Case2c:
        Create Connection, a Port and establish connection between new port, input port. User drags and drops a
        Connection shape onto the diagram and glues the same to a Block shape and a Port shape. A Port object is
        created if connected_ToRules is set to allow connection between ports. A Connection object is created. Port and
        Connection objects are pasted under respective paste targets. Connection is established between the Connection
        object and Port objects.
        
        Use Case2d: 
        Establish connection between existing Connection and Ports. User glues an existing Connection shape to Port
        shapes. Connection is established between the ports if connected_ToRules allows for connections between ports.
        
        Use Case2e: 
        Create new ports and establish connection between existing Connection and the Ports.
        User glues an existing Connection shape to Block shapes. Port objects are created and pasted under respective
        paste target objects if connected_ToRules allows for connections between ports. Connection is established
        between the Port objects and Connection object.
        
        Use Case2f: 
        Create new port and establish connection between existing Connection, Port and newly created Port. User glues
        an existing Connection shape to a Block shape and Port shape. A Port objects is created and pasted under it's
        paste target object if connected_ToRules allows for connections between ports. Connection is established
        between the Port objects and Connection object.
        
        Use Case2g: 
        Establish connection between existing Connection and Blocks. User glues an existing Connection shape to Block
        shapes. Connection is established between the Blocks and Connection object if connected_ToRules allows for
        connection between blocks.
        """
        return cls.execute_soa_method(
            method_name='createAndPaste',
            library='Diagramming',
            service_date='2014_06',
            service_name='DNDManagement',
            params={'inputData': inputData},
            response_cls=CreateAndPasteResponse,
        )

    @classmethod
    def createConnectionPortsAndConnect(cls, inputData: List[CreateConnectionPortsAndConnectInputInfo]) -> CreateConnectionPortsAndConnectResponse:
        """
        This operation creates ports and connection object given the port types, connection type. Pastes the new
        objects under respective paste targets and then connects the connection object to the ports. Connection, one or
        both ports can exist, in that case the missing objects are created and connection is established. Connection
        can also be established directly between blocks.
        
        Use cases:
        Use Case1: 
        Create a new object and paste it under the paste target. User drags and drops shapes onto the diagram. Shapes
        are mapped to specific types and a mapping file specifies this information. On drag and drop a new object of
        the desired type is created and is pasted on to the diagram root object. When the diagram is for a BOM
        structure the paste operation also returns a resultant runtime object [BOMLine] in addition to the newly
        created object. The runtime/newly created object is mapped to the shape on the diagram.
        
        Use Case2: 
        Connection Management
        
        Use Case2a: 
        Create Connection, Ports and establish connection between them. User drags and drops a Connection shape onto
        the diagram and glues the same to Block shapes. Two port objects are created and pasted under the Block objects
        if connected_ToRules is set to allow connection between ports; a Connection object is created and pasted under
        its paste target object. Connection is established between the Connection object and Port objects.
        
        Use Case2b:
        Create Connection, establish connection between input ports. User drags and drops a Connection shape onto the
        diagram and glues the same to Port shapes. A Connection object is created and pasted under the paste target.
        Connection is established between the Connection object and Port objects if connected_ToRules is set to allow
        connection between ports.
        
        Use Case2c: 
        Create Connection, a Port and establish connection between new    port, input port.
        User drags and drops a Connection shape onto the diagram and glues the same to a Block shape and a Port shape.
        A Port object is created if connected_ToRules is set to allow connection between ports. A Connection object is
        created. Port and Connection objects are pasted under respective paste targets. Connection is established
        between the Connection object and Port objects.
        
        Use Case2d: 
        Establish connection between existing Connection and Ports
        User glues an existing Connection shape to Port shapes. Connection is established between the ports if
        connected_ToRules allows for connections between ports.
        
        Use Case2e: 
        Create new ports and establish connection between existing Connection and the Ports.
        User glues an existing Connection shape to Block shapes. Port objects are created and pasted under respective
        paste target objects if connected_ToRules allows for connections between ports. Connection is established
        between the Port objects and Connection object.
        
        Use Case2f: 
        Create new port and establish connection between existing Connection, Port and newly created Port. User glues
        an existing Connection shape to a Block shape and Port shape. A Port objects is created and pasted under it's
        paste target object if connected_ToRules allows for connections between ports. Connection is established
        between the Port objects and Connection object.
        
        Use Case2g: 
        Establish connection between existing Connection and Blocks. User glues an existing Connection shape to Block
        shapes. Connection is established between the Blocks and Connection object if connected_ToRules allows for
        connection between blocks.
        """
        return cls.execute_soa_method(
            method_name='createConnectionPortsAndConnect',
            library='Diagramming',
            service_date='2014_06',
            service_name='DNDManagement',
            params={'inputData': inputData},
            response_cls=CreateConnectionPortsAndConnectResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Condition
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetSubscriptionConditionsResponse(TcBaseObj):
    """
    The response from the query request will include a list of the Conditions that have the correct signature to be
    used as Subscription criteria.
    
    :var conditions: A list of the Condition objects that can be used as part of a Subscription definition.
    :var serviceData: Returned service data.
    """
    conditions: List[Condition] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Internal.ProjectManagement._2007_06.ScheduleManagement import ResourceAssignmentLoadResponse, ScheduleModifyContainer, LoadResourceAssignmentContainer, LoadProgramViewResponse, LoadProgramViewContainer
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ScheduleManagementService(TcService):

    @classmethod
    def loadProgramView(cls, loadProgramViewContainer: LoadProgramViewContainer) -> LoadProgramViewResponse:
        """
        Load a program view
        Only one program view is accessed at a time. The program view itself consists of multiple schedules
        and is a top-level view of a program consisting of multiple schedules.
        """
        return cls.execute_soa_method(
            method_name='loadProgramView',
            library='Internal-ProjectManagement',
            service_date='2007_06',
            service_name='ScheduleManagement',
            params={'loadProgramViewContainer': loadProgramViewContainer},
            response_cls=LoadProgramViewResponse,
        )

    @classmethod
    def loadResourceAssignments(cls, loadResourceAssignmentContainer: LoadResourceAssignmentContainer) -> ResourceAssignmentLoadResponse:
        """
        Loads all the resource assignments for a given resource(s) in published and non-template schedules only.
        Besides this, if schedulesToAlwaysInclude parameter contains some schedules, they are also taken up.
        """
        return cls.execute_soa_method(
            method_name='loadResourceAssignments',
            library='Internal-ProjectManagement',
            service_date='2007_06',
            service_name='ScheduleManagement',
            params={'loadResourceAssignmentContainer': loadResourceAssignmentContainer},
            response_cls=ResourceAssignmentLoadResponse,
        )

    @classmethod
    def modifySchedule(cls, scheduleModifyContainer: ScheduleModifyContainer) -> ServiceData:
        """
        Updates all the affected scheduling objects based on the initial
        users request to the Application Interface.
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='modifySchedule',
            library='Internal-ProjectManagement',
            service_date='2007_06',
            service_name='ScheduleManagement',
            params={'scheduleModifyContainer': scheduleModifyContainer},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Internal.IcsAw._2017_12.Author import ClassSearchCriteria, FindClassificationsResponse
from typing import List
from tcsoa.base import TcService
from tcsoa.gen.BusinessObjects import WorkspaceObject


class AuthorService(TcService):

    @classmethod
    def findClassifications(cls, workspaceObjects: List[WorkspaceObject], searchCriterias: List[ClassSearchCriteria], classificationDataOptions: int) -> FindClassificationsResponse:
        """
        Finds classification information based on the input criteria.
        
        If a list of workspace objects are provided; then this operation finds out all the classification objects for
        the given workspace object.
        If a given workspace object is not classified; this operation finds and returns standalone classification
        object; which could then be linked to given workspace object.
        If a given workspace object is not classified and also system does not contain standalone classification object
        which could be linked to the workspace object; then this operation will return classification class hierarchy
        (children of "Classification Root" (ICM)) which could be used for browsing and classifying an object in a
        classification class of interest.
        
        If a search criteria is provided; then this operation searches for classification classes in classification
        hierarchy and returns their metadata information like class ID, name, class parents etc. 
        
        Note that this operation returns only the information that is asked through "classificationDataOptions"
        parameter.
        
        Use cases:
        This operation could be used when user needs to find classification objects (ICO) based on workspace objects.
        Each time a workspace object is classified in a classification class a classification object (ICO) object is
        created. After searching for all the classification objects corresponding to a workspace object, user can find
        more information about the classification(s) of a workspace object. The operation findClassification() can be
        used to get detailed information about the classification objects, classification classes where these
        classification objects belong. After getting information about classification objects, the user can also choose
        to modify or delete the object found by this operation using operation createOrUpdateClassificationObjects() or
        deleteClassificationObjects().
        
        The operation findClassification() can be used when user wants to search for classes in classification
        hierarchy by using any of class' properties. After searching for classification classes; the user can use that
        information to browse through a subset of classification hierarchy and classify a classifiable workspace object
        in a classification class of interest.
        """
        return cls.execute_soa_method(
            method_name='findClassifications',
            library='Internal-IcsAw',
            service_date='2017_12',
            service_name='Author',
            params={'workspaceObjects': workspaceObjects, 'searchCriterias': searchCriterias, 'classificationDataOptions': classificationDataOptions},
            response_cls=FindClassificationsResponse,
        )



from __future__ import annotations

from typing import Dict


"""
Map of string key names to string key values. This map is used to store option names and their corresponding option values.
"""
OptionKeyToOptionValueMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Internal.Manufacturing._2015_10.StructureManagement import AdditionalInfo
from tcsoa.gen.BusinessObjects import BusinessObject, Item, ItemRevision
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class AsyncCloningStructsInfo(TcBaseObj):
    """
    A list of AsyncCloningStructsInfo which contains VisStructureContext objects which are to be cloned in CPC. The
    structure contains additional cloning information as well.
    
    :var newName: The name of new structure to be created as a cloned structure in CPC.
    :var newDescription: The description of new structure to be created as a cloned structure in CPC.
    :var newId: Id of new structure to be created as a cloned structure in CPC.
    :var newRevId: The revision id of new structure to be created as a cloned structure in CPC.
    :var scopes: The list of  VisStructureContext objects to be cloned in CPC. It can be either root
    VisStructureContext object or multiple VisStructureContext objects within the same structure.
    :var cloningRule: The rule used for cloning. The value varies based on the structure type which is getting cloned.
    For example, the acceptable value for Mfg0MEPlantBOP is "ProjectPlantBOPTemplate".
    :var cloningParams: A map (string, string) representing cloning parameter and its value, such as
    ("CPC_carryOverICs","true") for carry over Incremental Changes. Other cloning parameters are "CPC_cloneSupressLine"
    and "CPC_createDIPA" for clone suppressed lines and create Dynamic In-Process Assembly (DIPA) respectively.
    """
    newName: str = ''
    newDescription: str = ''
    newId: str = ''
    newRevId: str = ''
    scopes: List[BusinessObject] = ()
    cloningRule: str = ''
    cloningParams: AsyncCloningParams = None


@dataclass
class AsyncCreateCPCInputInfo(TcBaseObj):
    """
    The input to the service operation is a structure which contains a list of VisStructureContext objects;  which are
    to be cloned and\or referred, cloning information, name and description of new CPC.
    
    :var cloningStructsInfo: A list of AsyncCloningStructsInfo which contains VisStructureContext objects which are to
    be cloned in CPC. The structure contains additional cloning information as well.
    :var releaseStatus: It contains the release status which gets associated with line(s) in the cloned structure if
    the line(s) doesn't have any release status in the original structure. The preference
    MEAlternativeDefaultReleaseStatus specifies the valid values.
    :var effectivityInfo: The information required to set effectivity for a list of cloned BOMLine objects.
    :var originalCCUid: The uid of the MECollaborationContext from which CPC is to be created.
    :var cpcName: Name of the CPC to be created.
    :var cpcDesc: Description of the CPC to be created.
    :var refStructures: List of VisStructureContext objects from the original CC that are going to be referred in the
    CPC.
    """
    cloningStructsInfo: List[AsyncCloningStructsInfo] = ()
    releaseStatus: str = ''
    effectivityInfo: AsyncEffectivityInfoInput = None
    originalCCUid: BusinessObject = None
    cpcName: str = ''
    cpcDesc: str = ''
    refStructures: List[BusinessObject] = ()


@dataclass
class AsyncEffectivityInfoInput(TcBaseObj):
    """
    The information required to set effectivity for a list of cloned BOMLine objects. Same as EffectivityInfoInput data
    type used in (createCollabPlanningContext) synchronous operation.
    
    :var endItem: Effectivity end Item.
    :var endItemRev: Effectivity end ItemRevision.
    :var unitRangeText: Effectivity unit range, for example "5-10". Multiple range can also be provided, for example
    "3,5-10,20-50". This range belongs to the end Item which is part of the structrue on which the unit effectivity is
    applied.
    Valid unit ranges are:
     -StartUnit
    -StartUnit - EndUnit
    -StartUnit - SO
    -StartUnit - UP
            where, StartUnit <= EndUnit 
    -StartUnit1 - EndUnit1, StartUnit2 - EndUnit2(Ex: 10 - 12, 15, 16 - UP)
            where StartUnit2 > EndUnit1.
    - All units are positive integers
    
    For example, the effectivity can be 3 - 5 where 3 is start unit and 5 is end unit.
    It can also be 3 - UP where 3 is start unit and "UP" means anything above the start unit. "UP" and "SO" indicates
    open ended effectivity.  "SO" is stock out means effectivity will be applicable till the stock out condition.
    :var dateRange: The list of effectivity date range. For instance, opened date range could be specified as "05 Jan -
    UP" and closed date range as "01 Jan - 03 Apr". The list may have any number of opened and closed date range. For
    example, If you select multiple dates such as start date 19th May 2015 and End date 22nd May 2015 , start date 19th
    June 2015 and end date 22nd June 2015, the date Range list have 4 values as below : 2015-02-19, 2015-05-22,
    2015-06-19,2015-06-22.
    :var openEndedStatus: Effectivity open ended status.
    - 0 : for closed unit range effectivity or date effectivity.
    - 1 : for opened unit range effectivity or date effectivity (UP).
    - 2 : for opened unit range effectivity or date effectivity stock out (SO).
    For example, if you select open ended effectivity such as start date 19th May 2015 and end date "UP", the dateRange
    list has 2015-05-19 with "openEndedStatus" value as "1".
    """
    endItem: Item = None
    endItemRev: ItemRevision = None
    unitRangeText: str = ''
    dateRange: List[datetime] = ()
    openEndedStatus: int = 0


@dataclass
class LinkStructuresInput(TcBaseObj):
    """
    This operation links or unlinks the BOMViews of the toplines of the two input structures by the given relation.
    
    :var sourceLine: The BOMLine of the source structure to be linked.
    :var targetLine: The BOMLine of the target structure to be linked.
    :var relationName: The name of the relation to link the structures.
    :var link: True to link structures, False to unlink structures.
    """
    sourceLine: BusinessObject = None
    targetLine: BusinessObject = None
    relationName: str = ''
    link: bool = False


@dataclass
class LinkStructuresResponse(TcBaseObj):
    """
    Response to hold linked structures.
    
    :var serviceData: The updated objects element will contain the connected PSBOMView objects. May also contain
    partial errors.
    :var additionalInfo: Future use
    """
    serviceData: ServiceData = None
    additionalInfo: AdditionalInfo = None


@dataclass
class ObjectAlignmentInput(TcBaseObj):
    """
    A list of ObjectAlignmentInput objects specifying the input structures.
    
    :var inputLine: The  line of structure to process.
    :var additionalInfo: Reserved for future use.
    """
    inputLine: BusinessObject = None
    additionalInfo: AdditionalInfo = None


@dataclass
class CompletenessCheckPartStructureResp(TcBaseObj):
    """
    Returns complete lines, incomplete lines and reasons for incomplete lines.
    
    :var completeLines: The list of BOMLline objects which are complete.
    :var incompleteLines: The list of BOMLine objects  which are evaluated as incomplete.
    :var incompleteLinesReason: The reasons the line is incomplete.A comma delimited string of reasons. Same size as
    incompleteLines.
    :var serviceData: Partial Errors
    :var additionalInfo: Reserved for future use.
    """
    completeLines: BusinessObject = None
    incompleteLines: BusinessObject = None
    incompleteLinesReason: str = ''
    serviceData: ServiceData = None
    additionalInfo: AdditionalInfo = None


@dataclass
class CreateDesignPartAlignmentInput(TcBaseObj):
    """
    List of CreateDesignPartAlignmentInput which specifies Design BOMLine, Part BOMLine and align mode .
    
    :var designLine: The Design BOMLine for alignment.
    :var partLine: The Part BOMLine for alignment.
    :var alignMode: The string representing align mode is "AssignDesign" | "NoAssign"
    :var additionalInfo: Future use.
    """
    designLine: BusinessObject = None
    partLine: BusinessObject = None
    alignMode: str = ''
    additionalInfo: AdditionalInfo = None


@dataclass
class DesignPartAlignmentInput(TcBaseObj):
    """
    Specifies Design and Part objects to verify.
    
    :var designLine: The input Design BOMLine. (Optional)
    :var partLine: The Part BOMLine. If no Design exist the service will issue an error.
    :var additionalInfo: Reserved for future use.
    """
    designLine: BusinessObject = None
    partLine: BusinessObject = None
    additionalInfo: AdditionalInfo = None


@dataclass
class DesignPartAlignmentResponse(TcBaseObj):
    """
    Returns service objects or partial errors.
    
    :var serviceData: Updated objects and partial error information.
    :var additionalInfo: Future use.
    """
    serviceData: ServiceData = None
    additionalInfo: AdditionalInfo = None


"""
A map (string, string) of the cloning parameters representing cloning parameter and its value. Same as CloningParams data type used in (createCollabPlanningContext) synchronous operation.
"""
AsyncCloningParams = Dict[str, str]



from tcsoa.gen.Classification._2009_10.services import ClassificationService as imp0
from tcsoa.gen.Classification._2016_03.services import ClassificationService as imp1
from tcsoa.gen.Classification._2007_01.services import ClassificationService as imp2
from tcsoa.gen.Classification._2011_06.services import ClassificationService as imp3
from tcsoa.gen.Classification._2016_09.services import ClassificationService as imp4
from tcsoa.gen.Classification._2015_10.services import ClassificationService as imp5
from tcsoa.gen.Classification._2011_12.services import ClassificationService as imp6
from tcsoa.gen.Classification._2015_03.services import ClassificationService as imp7
from tcsoa.base import TcService


class ClassificationService(TcService):
    autoComputeAttributes = imp0.autoComputeAttributes
    convertValues = imp1.convertValues
    createClassificationObjects = imp2.createClassificationObjects
    createOrUpdateKeyLOVs = imp0.createOrUpdateKeyLOVs
    deleteChildrenInHierarchy = imp3.deleteChildrenInHierarchy
    deleteClassificationObjects = imp2.deleteClassificationObjects
    findClassificationObjects = imp2.findClassificationObjects
    findClassifiedObjects = imp2.findClassifiedObjects
    findValues = imp4.findValues
    getAttributesForClasses = imp2.getAttributesForClasses
    getAttributesForClasses2 = imp0.getAttributesForClasses2
    getChildren = imp2.getChildren
    getChildrenExtended = imp1.getChildrenExtended
    getClassDefinitions = imp5.getClassDefinitions
    getClassDescriptions = imp2.getClassDescriptions
    getClassificationObjectInfo = imp6.getClassificationObjectInfo
    getClassificationObjects = imp2.getClassificationObjects
    getFileIds = imp2.getFileIds
    getKeyLOVs = imp2.getKeyLOVs
    getKeyLOVs2 = imp0.getKeyLOVs2
    getKeyLOVsForDependentAttributes = imp7.getKeyLOVsForDependentAttributes
    getLibraryHierarchy = imp3.getLibraryHierarchy
    getParents = imp2.getParents
    getPartFamilyTemplates = imp2.getPartFamilyTemplates
    search = imp2.search
    searchByInstanceId = imp2.searchByInstanceId
    searchClassesExtended = imp1.searchClassesExtended
    searchForClasses = imp2.searchForClasses
    updateClassificationObjects = imp2.updateClassificationObjects



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.Requirementsmanagement._2012_10.RequirementsManagement import MatchLineInputData, MatchedLineResponse, TraceabilityMatrixInfo1
from tcsoa.base import TcService
from tcsoa.gen.Internal.Requirementsmanagement._2012_02.RequirementsManagement import TraceabilityMatrixResponse


class RequirementsManagementService(TcService):

    @classmethod
    def getTraceabilityMatrix(cls, inputs: List[TraceabilityMatrixInfo1]) -> TraceabilityMatrixResponse:
        """
        This operation creates the traceability matrix between two BOM structures based on TraceLink relation by
        applying filter of selected trace link types. A matrix will be created according to the given TraceLink types
        instead of getting all TraceLink type relations. A matrix will be generated that shows the source structure
        objects as rows in the matrix and columns as target structure. In the trace matrix if there is a link between
        row (object from the source structure) and the column (object from the target structure) then link count will
        be shown in the respective cell for the TraceLink objects which are included in the filter types array. If
        filter is not applied then it will get the count of TraceLink objects. This operation can be performed between
        two different BOM structures or same structure. In this operation all children for selected source and target
        object will be loaded as rows and columns in the matrix and each cell defines the link between source row
        object and target column object. This matrix will show the results for TraceLink instances which are created
        from source object (object shown as rows) to target object (object shown as columns) and not vice versa.
        """
        return cls.execute_soa_method(
            method_name='getTraceabilityMatrix',
            library='Internal-Requirementsmanagement',
            service_date='2012_10',
            service_name='RequirementsManagement',
            params={'inputs': inputs},
            response_cls=TraceabilityMatrixResponse,
        )

    @classmethod
    def getMatchingLines(cls, inputs: List[MatchLineInputData]) -> MatchedLineResponse:
        """
        This operation searches for matching BOMLine instance(s) or GDELine instance(s) for a given ItemRevision in the
        given BOMWindow instances. For given ItemRevision, operation searches the matching BOMLine or GDELine in each
        BOM tree table, and returns matching BOMLine instances.
        """
        return cls.execute_soa_method(
            method_name='getMatchingLines',
            library='Internal-Requirementsmanagement',
            service_date='2012_10',
            service_name='RequirementsManagement',
            params={'inputs': inputs},
            response_cls=MatchedLineResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, BOMWindow, Awb0ProductContextInfo, Awb0SavedBookmark
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class BOMLineSelectionPath(TcBaseObj):
    """
    This is the ordered list of BOMLines that represents the selection path from the first level child to the selected
    BOMLine.
    
    :var selectionPath: An ordered list of BOMLines that represents the selection path from the first level child (the
    first index in the list) to the selected BOMLine (the last index in the list).  The top level is not included.
    """
    selectionPath: List[BOMLine] = ()


@dataclass
class ProductContextBOMWindowResponse(TcBaseObj):
    """
    This is the response for the createBOMWindowsFromContexts operation that creates BOMWindow objects from
    Awb0ProductContextInfo objects.
    
    :var bomWindowsAndSelections: A map (Awb0ProductContextInfo/BOMWindowAndSelections) of the input
    Awb0ProductContextInfo objects to the BOMWindowAndSelections objects created from the Awb0ProductContextInfo.
    :var serviceData: The SOA framework object containing objects that were created by the operation and error
    information.
    """
    bomWindowsAndSelections: ProductContextInfoBOMWindowMap = None
    serviceData: ServiceData = None


@dataclass
class BOMWindowAndSelections(TcBaseObj):
    """
    This is the container for a BOMWindow and the list of selected BOMLineSelectionPaths for the BOMLines on the
    BOMWindow.
    
    :var bomWindow: The BOMWindow object created from an Awb0SavedBookmark or Awb0ProductContextInfo.
    :var selections: A list of BOMLineSelectionPaths for the selected BOMLines on the BOMWindow.
    """
    bomWindow: BOMWindow = None
    selections: List[BOMLineSelectionPath] = ()


@dataclass
class SavedBookmarksBOMWindowsResponse(TcBaseObj):
    """
    This is the response for the createBOMWindowsFromBookmarks operation that creates BOMWindow objects from
    Awb0ProductSavedBookmark objects.
    
    :var bomWindowsAndSelections: A map (Awb0SavedBookmark/list of BOMWindowAndSelections objects)  of the input
    Awb0SavedBookmark objects to the list of BOMWindowAndSelections object that contains the BOMWindow and selected
    BOMLines created from the Awb0SavedBookmark.  Each saved Awb0SavedBookmark may contain muliple products, and each
    product will cause a unique BOMWindowAndSelections object to be created.  A list of BOMWindowAndSelections objects
    is supplied for each Awb0SavedBookmark.
    :var serviceData: The SOA framework object containing objects that were created by the operation and error
    information.
    """
    bomWindowsAndSelections: SavedBookmarkBOMWindowsMap = None
    serviceData: ServiceData = None


"""
A map of Awb0SavedBookmark objects to a list of BOMWindowAndSelections objects (Awb0SavedBookmark, list of BOMWindowAndSelections objects).
"""
SavedBookmarkBOMWindowsMap = Dict[Awb0SavedBookmark, List[BOMWindowAndSelections]]


"""
A map of Awb0ProductContextInfo objects to BOMWindowAndSelections objects (Awb0ProductContextInfo, BOMWindowAndSelections).
"""
ProductContextInfoBOMWindowMap = Dict[Awb0ProductContextInfo, BOMWindowAndSelections]



from __future__ import annotations

from tcsoa.gen.Internal.ActiveWorkspaceBom._2015_07.OccurrenceManagement import FindMatchingFilterResponse, FindMatchingFilterInput
from typing import List
from tcsoa.base import TcService


class OccurrenceManagementService(TcService):

    @classmethod
    def findMatchingFilters(cls, input: List[FindMatchingFilterInput]) -> FindMatchingFilterResponse:
        """
        This operation retrieves filters matching input search string.
        """
        return cls.execute_soa_method(
            method_name='findMatchingFilters',
            library='Internal-ActiveWorkspaceBom',
            service_date='2015_07',
            service_name='OccurrenceManagement',
            params={'input': input},
            response_cls=FindMatchingFilterResponse,
        )



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from typing import Dict


@dataclass
class DebugLoggingResponse(TcBaseObj):
    """
    The DebugLoggingResponse data type provides a response to the client of the debug logging flags that were set and
    the service data information.
    
    :var prevDebugLoggingFlags: Previous debug logging flags.
    :var serviceData: Service data information.
    """
    prevDebugLoggingFlags: Logflags = None
    serviceData: ServiceData = None


"""
A map (string, string) of logflags that enables debug logs.
"""
Logflags = Dict[str, str]



from tcsoa.gen.Internal_DocumentManagement._2013_05.services import DigitalSignatureService as imp0
from tcsoa.gen.Internal_DocumentManagement._2008_06.services import DispatcherManagementService as imp1
from tcsoa.gen.Internal_DocumentManagement._2008_06.services import DocumentControlService as imp2
from tcsoa.gen.Internal_DocumentManagement._2013_12.services import MarkupService as imp3
from tcsoa.gen.Internal_DocumentManagement._2020_12.services import AttributeExchangeService as imp4
from tcsoa.base import TcService


class DigitalSignatureService(TcService):
    cancelSign = imp0.cancelSign
    digitalSigningSaveTool = imp0.digitalSigningSaveTool
    isCheckOutForSign = imp0.isCheckOutForSign


class DispatcherManagementService(TcService):
    getConfigurations = imp1.getConfigurations


class DocumentControlService(TcService):
    getIRDCs = imp2.getIRDCs


class MarkupService(TcService):
    loadMarkups = imp3.loadMarkups
    saveMarkups = imp3.saveMarkups


class AttributeExchangeService(TcService):
    processAttrExchangeConfigurations = imp4.processAttrExchangeConfigurations



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.ProjectManagement._2009_10.ScheduleManagement import ScheduleModifyContainer, GenericAttributesContainer
from tcsoa.gen.Internal.ProjectManagement._2008_06.ScheduleManagement import ScheduleModifyContainer, GenericAttributesContainer
from typing import List
from tcsoa.gen.Internal.ProjectManagement._2007_06.ScheduleManagement import StringValContainer
from tcsoa.gen.Internal.ProjectManagement._2007_01.ScheduleManagement import StringValueContainer
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class KeyValuePair(TcBaseObj):
    """
    Simple key value pair struture
    
    :var key: the integer key - the parameterized index
    :var value: the string value - the parameterized value
    """
    key: int = 0
    value: str = ''


@dataclass
class TranslationDataContainer(TcBaseObj):
    """
    Internal data structure for moving POM objects between C/C++ and Java translator
    
    :var schedules: list of TranslationDataSchedule structure
    :var modelMetaData: Structure containing model classes
    :var preferences: list of string value containers of preferences
    :var requestData: request data
    """
    schedules: TranslationSchedules = None
    modelMetaData: TranslationModelMetaData = None
    preferences: TranslationPreferences = None
    requestData: TranslationDataRequest = None


@dataclass
class TranslationDataRequest(TcBaseObj):
    """
    Contain list of request structs
    
    :var genericContainers: List of generic containers
    :var scheduleModifyContainer: list Schedule modify container (there should always be only one of
    scheduleModifyContainer in the request. It was made into a vector in order to make the bounds 0 or more)
    """
    genericContainers: List[GenericAttributesContainer] = ()
    scheduleModifyContainer: List[ScheduleModifyContainer] = ()


@dataclass
class TranslationDataSchedule(TcBaseObj):
    """
    Contain list of POM objects
    
    :var dataObjects: List of POM objects
    """
    dataObjects: List[BusinessObject] = ()


@dataclass
class TranslationModelClass(TcBaseObj):
    """
    Model Class
    
    :var attribute: attribute meta data
    :var name: class name
    :var parentClassName: parent class name
    """
    attribute: List[StringValueContainer] = ()
    name: str = ''
    parentClassName: str = ''


@dataclass
class TranslationModelMetaData(TcBaseObj):
    """
    Schedule Schema
    
    :var modelClass: model class
    """
    modelClass: List[TranslationModelClass] = ()


@dataclass
class TranslationPreferences(TcBaseObj):
    """
    Contain preferences for the translation;
    SiteTimeZone,
    Default_Base_Calendar_Preference, 
    DefaultActualToSystemDate,
    SM_SCHEDULING_ENGINE_DATE,
    SM_View_CriticalPath.
    These preferences can have one of the types;
    TC_preference_all = 0;
    TC_preference_user = 1;
    TC_preference_role = 2;
    TC_preference_group = 3;
    TC_preference_site = 4;
    
    :var preferences: list of preferences
    """
    preferences: List[StringValContainer] = ()


@dataclass
class TranslationSchedules(TcBaseObj):
    """
    Contain list of ScheduleData structures
    
    :var translationDataSchedule: list of TranslationDataSchedule structures
    """
    translationDataSchedule: List[TranslationDataSchedule] = ()


@dataclass
class TranslatorException(TcBaseObj):
    """
    A structure containing translator exception IDs
    
    :var objectUid: the Uid of the object
    :var operation: The operation that genrated the exception.
    The type is suppose to be Teamcenter::Soa::ProjectManagement::_2009_10::ScheduleManagement::TranslatorOperation
    but this is generating error in SOA build (error published type being used in un-published type)
    :var errorNumber: the translator error number
    :var errorSeverity: The severity of the error from the translator; 
    information = 1; 
    warning =     2;
    error   =     3;         
    user_error =  4;
    :var errorString: the translator error string
    :var parameterValues: values for a parameterized error message
    """
    objectUid: str = ''
    operation: int = 0
    errorNumber: int = 0
    errorSeverity: int = 0
    errorString: str = ''
    parameterValues: List[KeyValuePair] = ()


@dataclass
class TranslatorResponseContainer(TcBaseObj):
    """
    Generic response structure
    
    :var translatorResponseContainerSub: list of sub translator response containers
    """
    translatorResponseContainerSub: List[TranslatorResponseContainerSub] = ()


@dataclass
class TranslatorResponseContainerSub(TcBaseObj):
    """
    A structure made up _0806 GenericAttributeContainers and TranslatorExceptions
    
    :var scheduleUid: the schedule uid
    :var genericAttributesContainer: list of GenericAttributeContainer structures
    :var translatorExceptions: list of translator exceptions
    """
    scheduleUid: str = ''
    genericAttributesContainer: List[GenericAttributesContainer] = ()
    translatorExceptions: List[TranslatorException] = ()


@dataclass
class TranslatorResponseScheduleModifyContainer(TcBaseObj):
    """
    Schedule modify response structure
    
    :var translatorResponseScheduleModifyContainerSub: list of sub translator schedule modify response containers
    """
    translatorResponseScheduleModifyContainerSub: List[TranslatorResponseScheduleModifyContainerSub] = ()


@dataclass
class TranslatorResponseScheduleModifyContainerSub(TcBaseObj):
    """
    structure used internally on the server side to share objects between C++ and Java
    
    :var scheduleUid: the schedule uid
    :var scheduleModifyContainer: list of schedule modify structures
    :var translatorExceptions: list of translator exceptions
    """
    scheduleUid: str = ''
    scheduleModifyContainer: List[ScheduleModifyContainer] = ()
    translatorExceptions: List[TranslatorException] = ()



from __future__ import annotations

from tcsoa.gen.Internal.Manufacturing._2015_10.StructureManagement import AdditionalInfo
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class AlternativeScopeForProductInputInfo(TcBaseObj):
    """
    Data required to create Fnd0MfgAlternativeScope for each product.
    
    :var productNode: A root BOMLine object  to create the Fnd0MfgAlternativeScope from.
    :var newAlternativeScpName: The name of Fnd0MfgAlternativeScope to be created.
    :var newAlternativeScpDesc: The description of the Fnd0MfgAlternativeScope to be created.
    :var containerInfo: A map (string,  list of BOMLine objects) containing the container name as key and list of
    BOMLine nodes which will be part of the container as value. Valid container names are "In-process Assembly
    Container", "Manufacturing Features Container" and "Connected To Container".
    """
    productNode: BusinessObject = None
    newAlternativeScpName: str = ''
    newAlternativeScpDesc: str = ''
    containerInfo: AlternativeScopeContainerInfo = None


@dataclass
class AlternativeScopeForProductResponse(TcBaseObj):
    """
    The response contains map where the key is input root BOMLine and the value is newly created
    Fnd0MfgAlternativeScope object.The following partial error may be returned :
    - 251076  : Alternative Scope creation failed for the given product structure.
    
    
    
    :var productNodeAlternativeScp: The map(BOMLine, BOMLine)  containing the input BOMLine node as key and the newly
    created Fnd0MfgAlternativeScope as value.
    :var serviceData: standard service data containing partial errors.
    """
    productNodeAlternativeScp: ProductNodeAndAlternativeScpMap = None
    serviceData: ServiceData = None


@dataclass
class PasteByRuleInfo(TcBaseObj):
    """
    Structure representing a source BOMLine and target BOMline pair.
    
    :var sourceLine: The source BOMline of type Mfg0BVROperation or type Mfg0BvrProcess to be pasted.
    :var targetLine: The target BOMline of type Mfg0BvrProcessStation from a PlantBOP to paste source BOMline to.
    :var additionalInfo: Currently not used.
    """
    sourceLine: BusinessObject = None
    targetLine: BusinessObject = None
    additionalInfo: AdditionalInfo = None


@dataclass
class PasteByRuleInfoInput(TcBaseObj):
    """
    Contains a  list of PasteByRuleInfo elements each representing a source and target line pair.
    
    :var sourceTargetLineInfo: List of structures of type representing source and target BOMline pairs.
    :var targetLine: Represents the target BOMLine in the PlantBOP. Can be provided when target is same for all source
    BOMLine objects.
    :var additionalInfo: Currently not used.
    """
    sourceTargetLineInfo: List[PasteByRuleInfo] = ()
    targetLine: BusinessObject = None
    additionalInfo: AdditionalInfo = None


@dataclass
class PasteByRuleResponse(TcBaseObj):
    """
    Structure representing created BOMLine objects and partial errors.
    
    :var createdLines: A list of  BOMLine objects created during the paste operation.  Can be NULL for some BOMLine
    objects if there is an error during creation of the new BOMLine. The index of createdLines matches the index of the
    sourceTargetLineInfo list.
    :var serviceData: The service data containing partial errors.
    :var additionalInfo: Currently not used.
    """
    createdLines: List[BusinessObject] = ()
    serviceData: ServiceData = None
    additionalInfo: AdditionalInfo = None


"""
A map contains the input product root node as key and the newly created Fnd0MfgAlternativeScope object as value.
"""
ProductNodeAndAlternativeScpMap = Dict[BusinessObject, BusinessObject]


"""
A map contains the container information. The key of the map is container name and the value is list of BOMLine objects.
"""
AlternativeScopeContainerInfo = Dict[str, List[BusinessObject]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ImanQuery
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExecuteSavedQueriesResponse(TcBaseObj):
    """
    Holds the response for executeSavedQueries.
    
    :var arrayOfResults: A set of SavedQueryResults data structures
    :var serviceData: serviceData
    """
    arrayOfResults: List[SavedQueryResults] = ()
    serviceData: ServiceData = None


@dataclass
class RetrieveSearchCriteriaResponse(TcBaseObj):
    """
    Holds a vector of SavedSearchCriteriaInfo and a ServiceData. 
     To be used as a response for operation "retrieveSearchCriteria"
    
    :var output: A vector of SavedSearchCriteriaInfo
    :var serviceData: Standard ServiceData
    """
    output: List[SaveSearchCriteriaInfo] = ()
    serviceData: ServiceData = None


@dataclass
class SaveSearchCriteriaInfo(TcBaseObj):
    """
    Holds the criteria information to be saved or retrieved for a 
    saved search, a collection of which is known as "My Saved Searches". 
    A saved search stores the user's entries for a particular saved query 
    execution so that it can be recalled and rerun in the future.
    
    :var searchName: A unique name for the saved search
    :var queryName: The name of the saved query associated with the saved search
    :var keys: Unique keys associated with the "values" below - each key represents a criteria field
    :var values: The values associated with the "keys" above
    :var resultsType: The results type associated with the saved search
    :var virtualFolderPath: The virtual folder the saved search belongs to
    """
    searchName: str = ''
    queryName: str = ''
    keys: List[str] = ()
    values: List[str] = ()
    resultsType: str = ''
    virtualFolderPath: str = ''


@dataclass
class SavedQueryInput(TcBaseObj):
    """
    The information about each Saved Query to be processed is provided 
    by way of the SavedQueryInput data structure.
    
    :var query: The saved query object to be executed.
    :var entries: Names of search criteria relevant to the query object.
    :var values: Values for the criteria.
    :var limitList: A list of objects (optional) against which the search is conducted,
    if the list is empty, search will be conducted against the database.
    :var limitListCount: The size of the limitList.
    :var maxNumToReturn: A specified maximum number of matches to be returned,
    0 means no limit.
    :var resultsType: The type of results expected from this operation:
    0 (top-level objects only),
    1 (top-level objects plus children: Hierarchical/Indented results),
    2 (default value as specified on the query object).
    :var maxNumToInflate: The number of objects in the returned result that must 
    include properties:
    (-1)(all),
    n (any positive integer less than or equal to the maxNumToReturn).
    """
    query: ImanQuery = None
    entries: List[str] = ()
    values: List[str] = ()
    limitList: List[BusinessObject] = ()
    limitListCount: int = 0
    maxNumToReturn: int = 0
    resultsType: int = 0
    maxNumToInflate: int = 0


@dataclass
class SavedQueryResults(TcBaseObj):
    """
    Results of Saved Query execution, number of objects returned and hierarchy information in a relation map for
    Hierarchical/Indented execution.  Hierarchical/Indented execution example1: If the query is: Find all Items where
    the Item Revisions have a Specification Dataset with name = "xyz1", then results will be comprised of subsets, each
    containing: an Item, a qualifying Item Revision and a qualifying Dataset. For each result subset, the corresponding
    entries in the relation map would be 0 (for root Item), 1 (for first-level Item Revision), 2 (for second-level
    Dataset).   Hierarchical/Indented execution example2: If the query is: Find all Item Revisions that have a
    Specification Dataset with name = "xyz1" AND Form of type Item Revision Master with user_data_1 = "xyz2", then the
    results will comprise of subsets, each containing: an Item Revision, a qualifying Dataset, a qualifying Form and a
    qualifying Item Revision Master. For each result subset, the corresponding entries in the relation map would be 0
    (for root Item Revision), 1 (for first-level Dataset), 1 (for first-level Form) and 2 (for second-level Item
    Revision Master).
    
    :var numOfObjects: The number of objects returned.
    :var objectsRelMap: Used only for Hierarchical/Indented execution. An integer array containing the hierarchy level
    information which maps 1-to-1 to the the returned list of objects which is ordered as subsets. The integer code in
    the relation map indicates if the corresponding object in the results set is a root object (=0), or a first-level
    child (=1), or a second-level child (=2) and so on. In the case of child levels, the level information is with
    respect to the immediately preceding root object entry in the mapping. The order of returns is relevant.
    :var objects: The objects returned from the search.
    """
    numOfObjects: int = 0
    objectsRelMap: List[int] = ()
    objects: List[BusinessObject] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, EPMTask
from tcsoa.gen.Internal.Workflow._2013_05.Workflow import SoaEPMAction, SoaEPMSupportingValues
from tcsoa.base import TcService


class WorkflowService(TcService):

    @classmethod
    def performActionAsync(cls, task: EPMTask, action: SoaEPMAction, comments: str, password: str, supportingValue: SoaEPMSupportingValues, supportingObject: BusinessObject) -> None:
        """
        The method signature of performActionAsync will be same as the sync performAction except the output is a void
        type. It will be used by the Asynchronous Service. This method performs workflow actions on a task. The actions
        that can be performed using this operation are: assign, start, complete, skip, suspend, resume, undo, perform,
        approve, reject, promote and demote. Note: Special case this operation can be used to set only comments without
        changing the decision for perform signoff task. To do that, use the 'EPM_no_action' as input for action
        argument and use the current decision for the supportingValue argument.
        
        Exceptions:
        >Teamcenter::Soa::Server::ServiceException:
        """
        return cls.execute_soa_method(
            method_name='performActionAsync',
            library='Internal-Workflow',
            service_date='2013_05',
            service_name='Workflow',
            params={'task': task, 'action': action, 'comments': comments, 'password': password, 'supportingValue': supportingValue, 'supportingObject': supportingObject},
            response_cls=None,
        )
